/*
 * some color space conversion functions
 *
 * arg1: destination pointer
 * arg2: source pinter
 * arg3: # if pixels
 *
 * FIXME: endian probems, some functions works for little only
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "colorspace.h"

/* ----------------------------------------------------------------------- */
/* lookup tables for yuv2rgb, plus init code                               */

static long *Cr_r_tab, *Cr_g_tab, *Cb_g_tab, *Cb_b_tab;

static long *red_rgb15, *green_rgb15, *blue_rgb15;
static long *red_rgb16, *green_rgb16, *blue_rgb16;
static long *red_rgb32, *green_rgb32, *blue_rgb32;

void
yuv_init(void)
{
    int i;
    
    Cr_r_tab = malloc(256*sizeof(int));
    Cr_g_tab = malloc(256*sizeof(int));
    Cb_g_tab = malloc(256*sizeof(int));
    Cb_b_tab = malloc(256*sizeof(int));

    red_rgb15   = (long*)malloc(768*sizeof(long)) + 256;
    green_rgb15 = (long*)malloc(768*sizeof(long)) + 256;
    blue_rgb15  = (long*)malloc(768*sizeof(long)) + 256;

    red_rgb16   = (long*)malloc(768*sizeof(long)) + 256;
    green_rgb16 = (long*)malloc(768*sizeof(long)) + 256;
    blue_rgb16  = (long*)malloc(768*sizeof(long)) + 256;

    red_rgb32   = (long*)malloc(768*sizeof(long)) + 256;
    green_rgb32 = (long*)malloc(768*sizeof(long)) + 256;
    blue_rgb32  = (long*)malloc(768*sizeof(long)) + 256;

    for (i=0; i<256; i++) {
	Cr_r_tab[i] =  (0.419/0.299) * (i-128);
	Cr_g_tab[i] = -(0.299/0.419) * (i-128);
	Cb_g_tab[i] = -(0.114/0.331) * (i-128);
	Cb_b_tab[i] =  (0.587/0.331) * (i-128);
    }

    for (i = 0; i < 256; i++) {
	red_rgb15[i]   = (i << 7)  & 0x007e00;
	red_rgb16[i]   = (i << 8)  & 0x00f800;
	red_rgb32[i]   = (i << 16) & 0xff0000;

	green_rgb15[i] = (i << 2)  & 0x0003e0;
	green_rgb16[i] = (i << 3)  & 0x0007e0;
	green_rgb32[i] = (i << 8)  & 0x00ff00;

	blue_rgb15[i]  = (i >> 3)  & 0x00001f;
	blue_rgb16[i]  = (i >> 3)  & 0x00001f;
	blue_rgb32[i]  =  i        & 0x0000ff;
    }

    for (i = 0; i < 256; i++) {
	red_rgb15[i-256] = red_rgb15[0];
	red_rgb16[i-256] = red_rgb16[0];
	red_rgb32[i-256] = red_rgb32[0];
	red_rgb15[i+256] = red_rgb15[255];
	red_rgb16[i+256] = red_rgb16[255];
	red_rgb32[i+256] = red_rgb32[255];

	green_rgb15[i-256] = green_rgb15[0];
	green_rgb16[i-256] = green_rgb16[0];
	green_rgb32[i-256] = green_rgb32[0];
	green_rgb15[i+256] = green_rgb15[255];
	green_rgb16[i+256] = green_rgb16[255];
	green_rgb32[i+256] = green_rgb32[255];

	blue_rgb15[i-256] = blue_rgb15[0];
	blue_rgb16[i-256] = blue_rgb16[0];
	blue_rgb32[i-256] = blue_rgb32[0];
	blue_rgb15[i+256] = blue_rgb15[255];
	blue_rgb16[i+256] = blue_rgb16[255];
	blue_rgb32[i+256] = blue_rgb32[255];
    }
}

/* ----------------------------------------------------------------------- */
/* anything to 32k HiColor                                                 */

void
rgb16_to_rgb15(void *d, void *s, int p, int w)
{
    unsigned short *dest = d;
    unsigned short *src  = s;
    int             i    = p;
    
    while (i--) {
	*(dest++) = ((*src & 0xffc0) >> 1) | (*src & 0x001f);
	src++;
    }
}

/* ----------------------------------------------------------------------- */
/* anything to 64k HiColor                                                 */

void
gray_to_rgb16(void *d, void *s, int p, int w)
{
    unsigned short *dest = d;
    unsigned char  *src  = s;
    int             i    = p;
    unsigned short  h;

    while (i--) {
	h         = *(src++) & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);
    }
}

void
rgb32_to_rgb16(void *d, void *s, int p, int w)
{
    unsigned short *dest = d;
    unsigned long  *src  = s;
    int             i    = p;
    register unsigned long  h2;

    while (i--) {
	h2  = *(src++);
	*(dest++) = 
	    ((h2 & 0xf80000) >> 8) |
	    ((h2 & 0x00fc00) >> 5) |
	    ((h2 & 0x0000f8) >> 3);
    }
}

void
yuv422_to_rgb16(void *d, void *s, int p, int w)
{
    unsigned short *dest = d;
    unsigned char  *src  = s;
    int             i    = p;
    unsigned short  h;

    while (i--) {
	h         = *src & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);
	src += 2;
    }
}

void
yuv411_to_rgb16(void *d, void *s, int p, int w)
{
    unsigned short *dest = d;
    unsigned char  *src  = s;
    int             i    = p;
    unsigned short  h;

    while (i--) {
	h         = src[ 1] & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);
	h         = src[ 3] & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);
	h         = src[ 5] & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);
	h         = src[ 7] & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);

	h         = src[ 8] & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);
	h         = src[ 9] & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);
	h         = src[10] & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);
	h         = src[11] & 0xf8;
	*(dest++) = (h << 8) | (h << 3) | (h >> 3);
	
	src += 12;
    }
}

void
yuv411p_to_rgb16(void *d, void *s, int p, int w)
{
    unsigned short *dest1 = (unsigned short*)d;
    unsigned short *dest2 = (unsigned short*)d+w;
    unsigned char  *lum1  = (unsigned char*)s;
    unsigned char  *lum2  = (unsigned char*)s+w;
    unsigned char  *cb    = (unsigned char*)s+p;
    unsigned char  *cr    = (unsigned char*)s+p+p/4;
    int             cr_r, cr_g, cb_g, cb_b, l, x;

    while (p > 0) {
	for (x = w/2; x; x--) {
	    cr_r = Cr_r_tab[*cr];
            cr_g = Cr_g_tab[*cr];
            cb_g = Cb_g_tab[*cb];
            cb_b = Cb_b_tab[*cb];
	    cr++, cb++;

	    l = *(lum1++);
	    *(dest1++) = red_rgb16[l+cr_r] |
		green_rgb16[l+cr_g+cb_g] | blue_rgb16[l+cb_b];
	    l = *(lum1++);
	    *(dest1++) = red_rgb16[l+cr_r] |
		green_rgb16[l+cr_g+cb_g] | blue_rgb16[l+cb_b];

	    l = *(lum2++);
	    *(dest2++) = red_rgb16[l+cr_r] |
		green_rgb16[l+cr_g+cb_g] | blue_rgb16[l+cb_b];
	    l = *(lum2++);
	    *(dest2++) = red_rgb16[l+cr_r] |
		green_rgb16[l+cr_g+cb_g] | blue_rgb16[l+cb_b];
	}
	lum1  += w;
	lum2  += w;
	dest1 += w;
	dest2 += w;
	p     -= 2*w;
    }
}

void
yuv422p_to_rgb16(void *d, void *s, int p, int w)
{
    unsigned short *dest  = (unsigned short*)d;
    unsigned char  *lum   = (unsigned char*)s;
    unsigned char  *cb    = (unsigned char*)s+p;
    unsigned char  *cr    = (unsigned char*)s+p+p/2;
    int             cr_r, cr_g, cb_g, cb_b, l, x;

    while (p > 0) {
	for (x = w/2; x; x--) {
	    cr_r = Cr_r_tab[*cr];
            cr_g = Cr_g_tab[*cr];
            cb_g = Cb_g_tab[*cb];
            cb_b = Cb_b_tab[*cb];
	    cr++, cb++;

	    l = *(lum++);
	    *(dest++) = red_rgb16[l+cr_r] |
		green_rgb16[l+cr_g+cb_g] | blue_rgb16[l+cb_b];
	    l = *(lum++);
	    *(dest++) = red_rgb16[l+cr_r] |
		green_rgb16[l+cr_g+cb_g] | blue_rgb16[l+cb_b];
	}
	p     -= w;
    }
}

/* ----------------------------------------------------------------------- */
/* anything to 24bit TrueColor                                             */

void
rgb15_to_rgb24(void *d, void *s, int p, int w)
{
    unsigned char  *dest = d;
    unsigned short *src  = s;
    int             i    = p;
    register unsigned short h;

    while (i--) {
	h         = *(src++);
	*(dest++) = (h & 0x7c00) >> 7;
	*(dest++) = (h & 0x03e0) >> 3;
	*(dest++) = (h & 0x001f) << 3;
    }
}

void
rgb16_to_rgb24(void *d, void *s, int p, int w)
{
    unsigned char  *dest = d;
    unsigned short *src  = s;
    int             i    = p;
    register unsigned short h;

    while (i--) {
	h         = *(src++);
	*(dest++) = (h & 0xf800) >> 8;
	*(dest++) = (h & 0x07e0) >> 3;
	*(dest++) = (h & 0x001f) << 3;
    }
}

void
rgb32_to_rgb24(void *d, void *s, int p, int w)
{
    unsigned char  *dest = d;
    unsigned char  *src  = s;
    int             i    = p;

    while (i--) {
	*(dest++) = src[2];
	*(dest++) = src[1];
	*(dest++) = src[0];
	src += 4;
    }
}

void
rgb32_to_bgr24(void *d, void *s, int p, int w)
{
    unsigned char  *dest = d;
    unsigned char  *src  = s;
    int             i    = p;

    while (i--) {
	*(dest++) = src[0];
	*(dest++) = src[1];
	*(dest++) = src[2];
	src += 4;
    }
}

/* ----------------------------------------------------------------------- */
/* anything to 32bit TrueColor                                             */

void
gray_to_rgb32(void *d, void *s, int p, int w)
{
    unsigned long  *dest = d;
    unsigned char  *src  = s;
    int             i    = p;
    unsigned char   h;

    while (i--) {
	h         = *(src++);
	*(dest++) = h | (h << 8) | (h << 16);
    }
}

void
rgb15_to_rgb32(void *d, void *s, int p, int w)
{
    unsigned char  *dest = d;
    unsigned short *src  = s;
    int             i    = p;
    register unsigned short h;

    while (i--) {
	h         = *(src++);
	*(dest++) =
	    ((h & 0x7c00) << 9) |
	    ((h & 0x03e0) << 6) |
	    ((h & 0x001f) << 3);
    }
}

void
rgb16_to_rgb32(void *d, void *s, int p, int w)
{
    unsigned long  *dest = d;
    unsigned short *src  = s;
    int             i    = p;
    register unsigned short h;

    while (i--) {
	h         = *(src++);
	*(dest++) =
	    ((h & 0xf800) << 8) |
	    ((h & 0x07e0) << 5) |
	    ((h & 0x001f) << 3);
    }
}


void
yuv411p_to_rgb32(void *d, void *s, int p, int w)
{
    unsigned long  *dest1 = (unsigned long*)d;
    unsigned long  *dest2 = (unsigned long*)d+w;
    unsigned char  *lum1  = (unsigned char*)s;
    unsigned char  *lum2  = (unsigned char*)s+w;
    unsigned char  *cb    = (unsigned char*)s+p;
    unsigned char  *cr    = (unsigned char*)s+p+p/4;
    int             cr_r, cr_g, cb_g, cb_b, l, x;

    while (p > 0) {
	for (x = w/2; x; x--) {
	    cr_r = Cr_r_tab[*cr];
            cr_g = Cr_g_tab[*cr];
            cb_g = Cb_g_tab[*cb];
            cb_b = Cb_b_tab[*cb];
	    cr++, cb++;

	    l = *(lum1++);
	    *(dest1++) = red_rgb32[l+cr_r] |
		green_rgb32[l+cr_g+cb_g] | blue_rgb32[l+cb_b];
	    l = *(lum1++);
	    *(dest1++) = red_rgb32[l+cr_r] |
		green_rgb32[l+cr_g+cb_g] | blue_rgb32[l+cb_b];

	    l = *(lum2++);
	    *(dest2++) = red_rgb32[l+cr_r] |
		green_rgb32[l+cr_g+cb_g] | blue_rgb32[l+cb_b];
	    l = *(lum2++);
	    *(dest2++) = red_rgb32[l+cr_r] |
		green_rgb32[l+cr_g+cb_g] | blue_rgb32[l+cb_b];
	}
	lum1  += w;
	lum2  += w;
	dest1 += w;
	dest2 += w;
	p     -= 2*w;
    }
}

void
yuv422p_to_rgb32(void *d, void *s, int p, int w)
{
    unsigned long  *dest  = (unsigned long*)d;
    unsigned char  *lum   = (unsigned char*)s;
    unsigned char  *cb    = (unsigned char*)s+p;
    unsigned char  *cr    = (unsigned char*)s+p+p/2;
    int             cr_r, cr_g, cb_g, cb_b, l, x;

    while (p > 0) {
	for (x = w/2; x; x--) {
	    cr_r = Cr_r_tab[*cr];
            cr_g = Cr_g_tab[*cr];
            cb_g = Cb_g_tab[*cb];
            cb_b = Cb_b_tab[*cb];
	    cr++, cb++;

	    l = *(lum++);
	    *(dest++) = red_rgb32[l+cr_r] |
		green_rgb32[l+cr_g+cb_g] | blue_rgb32[l+cb_b];
	    l = *(lum++);
	    *(dest++) = red_rgb32[l+cr_r] |
		green_rgb32[l+cr_g+cb_g] | blue_rgb32[l+cb_b];
	}
	p     -= w;
    }
}
