/* 
    sound mixer for Bt848 frame grabber driver

    Copyright (C) 1996,97 Marcus Metzler (mocm@thp.uni-koeln.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * mixer.cpp
 */

#include <stream.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "mixer.h"
#include "wintv.h"

char *names[] = SOUND_DEVICE_NAMES;
char *labels[] = SOUND_DEVICE_LABELS;

#ifdef myDEBUG
#define sDEBUG
#endif
              
mixer::mixer(const char *filename, int device)
{
  int i;
  
  dev=-1;
  if (-1 == (mixer_device = open(filename,O_RDONLY))) {
    cerr << "mixer: error mixer open" << endl;
    return;
  }
  if (-1 == ioctl(mixer_device,MIXER_READ(SOUND_MIXER_DEVMASK),&devmask)) {
    cerr << "mixer: error mixer read devmask" << endl;
    return;
  }
  for (i = 0; i < SOUND_MIXER_NRDEVICES; i++) {
    if ((1<<i) & devmask && strcasecmp(names[i],names[device]) == 0)
      if (-1 == ioctl(mixer_device,MIXER_READ(i),&volume)) {
	cerr << "mixer: error mixer read volume" << endl;
	return;
      } else {
	dev = i;
	muted = 0;
      }
  }
  if (-1 == dev) {
    cerr << "mixer: cannot find mixer channel " << labels[device] << endl;
    cerr << "mixer: aviable channels are: " << endl;
    for (i = 0; i < SOUND_MIXER_NRDEVICES; i++)
      if ((1<<i) & devmask)
	cerr << "mixer: " << labels[i] << "  " << i << endl;
  }
  
  resetNextChannelName();

  return;
}


mixer::~mixer()
{ 
  close(mixer_device); 
  dev=-1; 

}

int mixer::name2channel(const char *_devName)
{
  int i;
  for (i = 0; i < SOUND_MIXER_NRDEVICES; i++) {
    if (strcasecmp(names[i],_devName) == 0)
      return(i);
  } 
  return(-1);
}

void mixer::resetNextChannelName()
{
  first=-1;
}

char *mixer::getNextChannelName(int *nr)
{
  first++;
  if (first >= SOUND_MIXER_NRDEVICES) { *nr=0; return(NULL); }
  while ( (!((1<<first) & devmask)) && (first < SOUND_MIXER_NRDEVICES)) first++;
  if (first >= SOUND_MIXER_NRDEVICES) { *nr=0; return(NULL); }
#ifdef sDEBUG
  cerr << "Debug: mixer: name " << first << " " << labels[first] << endl;
#endif
  *nr=first;
  return(labels[first]);
}

void mixer::set_vol(int l, int r=-1)
{
  // if (devmask & dev) {
    l   &= 0x7f;
    l = l | (l << 8);;
    m.l=l;
    if (r<0) m.r=l;
    else m.r=r;

    ioctl(mixer_device, MIXER_WRITE(dev), &(m));

    /*
#ifdef sDEBUG
    cerr << "Debug: sound: set " << labels[dev] << " to " 
	 << (int) m.l << ", " << (int) m.r << endl;
#endif
*/

    // }
}

void mixer::get_vol(int *v, int lr=LEFT){
  
  if (devmask & 1) {
    if (lr==LEFT) *v=m.l;
    else *v=m.r;
  } else {
    if (lr==LEFT) *v=0;
    else *v=0;    
  }
  
}




