#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <linux/string.h> // for memcpy

#include "rawvideoclip.h"

extern char graberror[256];
#define GRAB_PERROR(action,file) \
  {sprintf(graberror,"%s: %s: %s",action,file,strerror(errno));return -1;}

#define TRAP(txt) fprintf(stderr,"%s:%d:%s\n",__FILE__,__LINE__,txt);exit(1);


/* ----------------------------------------------------------------------- */

rawVideoClip::rawVideoClip():videoClip()
{
  strcpy(ext,".raw");
  strcpy(name,"RAW video file");
  video_formats=VIDEO_GRAY8 | VIDEO_RGB08 | VIDEO_RGB15 | VIDEO_RGB16 | VIDEO_RGB32 | VIDEO_YCrCb_411p | VIDEO_YCrCb_422p;
  audio_formats=AUDIO_NONE;
  
}

int rawVideoClip::open(char *filename, struct MOVIE_PARAMS *par)
{
  frames=0;
  strcpy(file,filename);
  if (strstr(file,ext) == NULL) strcat(file,ext);
  strcpy(par->extension,ext);
  
  memcpy(&params,par,sizeof(struct MOVIE_PARAMS));
  switch (par->video_format) {
  case VIDEO_RGB15:
  case VIDEO_RGB16:
    frame_bytes  = par->width * par->height *2;
    break;
  case VIDEO_RGB24:
    frame_bytes  = par->width * par->height *3;
    break;
  case VIDEO_RGB32:
    frame_bytes  = par->width * par->height *4;
    break;
  case VIDEO_GRAY8:
    frame_bytes  = par->width * par->height;
    break;
  case VIDEO_YCrCb_411p:
    frame_bytes  = par->width * par->height *3/2;
    break;
  case VIDEO_YCrCb_422p:
    frame_bytes  = par->width * par->height *2;
    break;
  default:
    TRAP("unsupported video format");
  }
  if (-1 == (fd = ::open(filename,O_CREAT | O_RDWR | O_TRUNC, 0666)))
    GRAB_PERROR("open",filename);

  return 0;
}

int rawVideoClip::writeframe(void *data)
{
    if (-1 == write(fd,data,frame_bytes))
	GRAB_PERROR("write",file);
    frames++;
    return 0;
}

int rawVideoClip::close()
{
    ::close(fd);
    return 0;
}

