/* 
    kwintv using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <unistd.h>
#include <qdir.h>
#include <kapp.h>
#include <kconfig.h>
#include <rcfile.h>
#include <wintvscreen.h>

rcfile::rcfile() 
{
  capture=0;
  sound_on=1;
  volume_left=0;
  volume_right=0;
  sliders_att_startup=1;
  delay=DEFAULT_DELAY;
  col=DCOL;
  hue=DHUE;
  bri=DBRI;
  contr=DCONTRAST;
  norm=DNORM;
  in=DIN;
  freq=DFREQ;
  country_name=QString("europebc.chan");
  country_id=0;
  channelListName=QDir::homeDirPath()+QString("/")+QString(RCDIR)+QString("/")+QString(CHFILE);
  bttvDeviceName=QString(DDEV);
  ircDeviceName=QString("/dev/irc");
  ircDataName=QDir::homeDirPath()+QString("/")+QString(RCDIR)+QString("/")+QString("code.irc");
  automouse=1;
  autosave=1;
  autott=1;
  use_master_volume=0;
  use_tv_volume=0;
  screensaver=1;
  fullSize=0;
  mixerDeviceName=QString("/dev/mixer");
  mixerDevice=0;
  aspectRatio=0;
  autoar=0;
  vc_audioformat=vc_dimension=vc_audiocapture=0;
  vc_channels=1; // mono
  vc_format=VIDEO_RGB15;
  vc_fps=10;
  vc_bits=8;
  vc_rate=11025;
  vc_tempName=QString("./temp");
  vc_video_clip=VIDEOCLIP_AVI;
  vc_width=160;
  vc_height=120;
}


rcfile::~rcfile()
{
  
}


int rcfile::load()
{
  QDir rcdir(QDir::homeDirPath ()+QString("/")+QString(RCDIR));
  if (!rcdir.exists()) {
    rcdir.mkdir(rcdir.absPath());
    chown((const char*)rcdir.absPath(),getuid(),getgid());
  }

  KConfig *config=KApplication::getKApplication()->getConfig();

  sliders_att_startup=config->readNumEntry("sliders_att_startup",sliders_att_startup);
  capture=config->readNumEntry("capture",capture);
  sound_on=config->readNumEntry("mute",sound_on);
  volume_left=config->readNumEntry("vol_left",volume_left);
  volume_right=config->readNumEntry("vol_right",volume_right);
  col=config->readNumEntry("color",col);
  hue=config->readNumEntry("hue",hue);
  contr=config->readNumEntry("contrast",contr);
  bri=config->readNumEntry("brightness",bri);
  norm=config->readNumEntry("norm",norm);
  in=config->readNumEntry("input",in);
  freq=config->readNumEntry("frequency",freq);
  country_name=config->readEntry("country_name",country_name);
  country_id=config->readNumEntry("country_id",country_id);
  channelListName=config->readEntry("channelListName",channelListName);
  bttvDeviceName=config->readEntry("bttvDeviceName",bttvDeviceName);
  ircDeviceName=config->readEntry("ircDeviceName",ircDeviceName);
  ircDataName=config->readEntry("ircDataName",ircDataName);
  automouse=config->readNumEntry("automouse",automouse);
  autosave=config->readNumEntry("autosave",autosave);
  autott=config->readNumEntry("autott",autott);
  use_master_volume=config->readNumEntry("use_master_volume",use_master_volume);
  screensaver=config->readNumEntry("screensaver",screensaver);
  fullSize=config->readNumEntry("fullsize",fullSize);
  mixerDeviceName=config->readEntry("mixerDeviceName",mixerDeviceName);
  mixerDevice=config->readNumEntry("mixerDevice",mixerDevice);
  aspectRatio=config->readNumEntry("aspect_ratio",aspectRatio);
  autoar=config->readNumEntry("autoar",autoar);
  use_tv_volume=config->readNumEntry("use_tv_volume",use_tv_volume);
  vc_audioformat=config->readNumEntry("vc_audioformat",vc_audioformat);
  vc_dimension=config->readNumEntry("vc_dimension",vc_dimension);
  vc_format=config->readNumEntry("vc_format",vc_format);
  vc_audiocapture=config->readNumEntry("vc_audiocapture",vc_audiocapture);
  vc_fps=config->readNumEntry("vc_fps",vc_fps);
  vc_channels=config->readNumEntry("vc_channels",vc_channels);
  vc_bits=config->readNumEntry("vc_bits",vc_bits);
  vc_rate=config->readNumEntry("vc_rate",vc_rate);
  vc_tempName=config->readEntry("vc_tempName",vc_tempName);
  vc_video_clip=config->readNumEntry("vc_video_clip",vc_video_clip);
  vc_height=config->readNumEntry("vc_height",vc_height);
  vc_width=config->readNumEntry("vc_width",vc_width);
  return(0); 
}

int rcfile::save()
{
  KConfig *config=KApplication::getKApplication()->getConfig();

  config->writeEntry("capture",capture);
  config->writeEntry("mute",sound_on);
  config->writeEntry("vol_left",volume_left);
  config->writeEntry("vol_right",volume_right);
  config->writeEntry("color",col);
  config->writeEntry("hue",hue);
  config->writeEntry("contrast",contr);
  config->writeEntry("brightness",bri);
  config->writeEntry("norm",norm);
  config->writeEntry("input",in);
  config->writeEntry("frequency",freq);
  config->writeEntry("country_name",country_name);
  config->writeEntry("country_id",country_id);
  config->writeEntry("channelListName",channelListName);
  config->writeEntry("bttvDeviceName",bttvDeviceName);
  config->writeEntry("ircDeviceName",ircDeviceName);
  config->writeEntry("ircDataName",ircDataName);
  config->writeEntry("automouse",automouse);
  config->writeEntry("autosave",autosave);
  config->writeEntry("autott",autott);
  config->writeEntry("use_master_volume",use_master_volume);
  config->writeEntry("screensaver",screensaver);
  config->writeEntry("fullsize",fullSize);
  config->writeEntry("mixerDeviceName",mixerDeviceName);
  config->writeEntry("mixerDevice",mixerDevice);
  config->writeEntry("aspect_ratio",aspectRatio);
  config->writeEntry("autoar",autoar);
  config->writeEntry("use_tv_volume",use_tv_volume);
  config->writeEntry("sliders_att_startup",sliders_att_startup);
  config->writeEntry("vc_audioformat",vc_audioformat);
  config->writeEntry("vc_dimension",vc_dimension);
  config->writeEntry("vc_format",vc_format);
  config->writeEntry("vc_audiocapture",vc_audiocapture);
  config->writeEntry("vc_fps",vc_fps);
  config->writeEntry("vc_channels",vc_channels);
  config->writeEntry("vc_bits",vc_bits);
  config->writeEntry("vc_rate",vc_rate);
  config->writeEntry("vc_tempName",vc_tempName);
  config->writeEntry("vc_video_clip",vc_video_clip);
  config->writeEntry("vc_height",vc_height);
  config->writeEntry("vc_width",vc_width);
  return(0);
}







