#ifndef videoclip_h
#define videoclip_h

#include <stdlib.h>

#define VIDEO_RGB08          1
#define VIDEO_RGB15          2
#define VIDEO_RGB16          4
#define VIDEO_RGB24          8
#define VIDEO_RGB32         16
#define VIDEO_GRAY8         32
#define VIDEO_YCrCb_422     64
#define VIDEO_YCrCb_422p   128      /* planar - not working yet */
#define VIDEO_YCrCb_411    256
#define VIDEO_YCrCb_411p   512

#define AUDIO_NONE      0
#define AUDIO_PCM       1
#define AUDIO_ALAW      2
#define AUDIO_MULAW     4

static char graberror[256];

/* ------------------------------------------------------------------------- */

struct FORMAT_NAMES {
    char  *name;
    int   format;
};

struct VIDEO_SIZE {
    char  *name;
    int   width;
    int   height;
};

/* examples for ...
struct FORMAT_NAMES format_names[] = {
    { "RGB 8",             VIDEO_RGB08      },
    { "RGB 15",            VIDEO_RGB15      },
    { "RGB 16",            VIDEO_RGB16      },
    { "RGB 24",            VIDEO_RGB24      },
    { "RGB 32",            VIDEO_RGB32      },
    { "GRAY 8",            VIDEO_GRAY8      },
    { "YUV 4:2:2",         VIDEO_YCrCb_422  },
    { "YUV 4:2:2 planar",  VIDEO_YCrCb_422p },
    { "YUV 4:1:1",         VIDEO_YCrCb_411  },
    { "YUV 4:1:1 planar",  VIDEO_YCrCb_411p },
    { NULL, 0 }
};

struct VIDEO_SIZE video_size[] = {
    { "PAL",        768, 576 },
    { "PAL/4",      384, 288 },
    { "PAL/16",     192, 144 },
    { "CIF",        352, 288 },
    { "QCIF",       176, 144 },
    { NULL, 0, 0 }
};
*/

/* ------------------------------------------------------------------------- */
/* movie output interface                                                    */

struct MOVIE_PARAMS {
    int video_clip;
    char filename[500];
    char extension[20];
    /* video */
    int video_format;
    int width, height;          /* size */
    int fps;                    /* frames per second */
    /* audio */
    int audio_format;
    int channels;               /* 0 = no sound, 1 = mono, 2 = stereo */
    int bits;                   /* 8/16 */
    int rate;                   /* sample rate in kHz (11025 etc) */
};


/* ------------------------------------------------------------------------- */

class videoClip
{
public:
  videoClip() { name=(char*)malloc(255); ext=(char*)malloc(255); }
  ~videoClip() { free(name); free(ext); }

  virtual int open(char *filename, struct MOVIE_PARAMS *par) = 0;
  virtual int writeframe(void *data) = 0;
  virtual int writesound(void *data, int datasize) = 0;
  virtual int close() = 0;

  virtual int nframes() = 0;
  struct MOVIE_PARAMS *parameter() { return &params; }
    
  /* ------------------------------------------------------------------------- */
  //private:
  /* file handle */
  int fd;
  char file[256];
  
  struct MOVIE_PARAMS params;
  
  /* audio stuff */
  int have_audio;

  int frames;

  char  *name;                /* format */
  char  *ext;                 /* extention */
  int   video_formats;        /* what we can handle */
  int   audio_formats;
};


#endif
