//-> Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)
/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
//<-

//-> definitions
//#define withTVscreen
//<-

//-> includes
#include <stdio.h>
#include <stdlib.h>

#include "wintv.h"
#include "xrwevent.h"
#include "wintvmain.h"  // immer in dieser Reihenfolge...
#include "wintvchannel.h"
#include "wintvvclip.h"

#include <kapp.h>

// X11 stuff
#include <X11/Xlib.h>
#ifdef X11DEBUG
#include "xeventtype.h"
#endif
//<-


//-> class TVApplication: public THEAPPLICATION
#define THEAPPLICATION KApplication

class TVApplication: public THEAPPLICATION
{
public:
  TVApplication( int & argc, char ** argv ):THEAPPLICATION( argc, argv ) {   } 
  //~TVApplication() {};

  bool x11EventFilter( XEvent * );
  void setFilterObject(QObject *theObject) { winTVscreenObject = theObject; } 

private:
  QObject *winTVscreenObject;
};
//<-

//-> bool TVApplication::x11EventFilter( XEvent *event)
bool TVApplication::x11EventFilter( XEvent *event )
{
  Window checkw1,checkw2,wwin;

  wwin=DefaultRootWindow(qt_xdisplay());
  checkw1=event->xmap.event;
  checkw2=event->xany.window; 

  if (checkw1 == wwin) {

    X11RWEvent *xrwe = new X11RWEvent(XRWEvent,
				    event->type,
				    event);
    postEvent(winTVscreenObject,xrwe);
    
#ifdef X11DEBUG
    debug("x11EF cwin1 xmap type: %d = %s",event->type,events[event->type]);
#endif

  return TRUE;
  }

  if (checkw2 == wwin) {
#ifdef X11DEBUG
  debug("x11EF cwin2 xmap type: %d = %s",event->type,events[event->type]);
#endif
  return TRUE; 
  }

  /*
 if ((checkw2 == wwin) || (checkw1 == wwin)) {
#ifdef myDEBUG
   debug("x11EventFilter: rootwindow event %d",wwin);
#endif
}
*/


 return FALSE;
}
//<-

//-> void myMessageOutput( QtMsgType type, const char *msg )
void myMessageOutput( QtMsgType type, const char *msg )
{
  switch ( type ) {
  case QtDebugMsg:
    fprintf( stderr, "Debug: %s\n", msg );
    break;
  case QtWarningMsg:
    fprintf( stderr, "Warning: %s\n", msg );
    break;
  case QtFatalMsg:
    fprintf( stderr, "Fatal: %s\n", msg );
    abort();                        // dump core on purpose
  }
}
//<-

//-> int main( int argc, char **argv )
//
int main( int argc, char **argv )
{
  qInstallMsgHandler( myMessageOutput );

  THEAPPLICATION::setColorSpec( THEAPPLICATION::ManyColor ); 
  TVApplication winTVApplication( argc, argv );

#ifdef myDEBUG
  debug("stage 1 ...");
#endif

  /////////////////////////////////////////////////////////////////////////////
  // load rc-file
  rcfile *rc=new rcfile();
  if (rc->load()) fatal("hardcore error 00001");

#ifdef myDEBUG
  debug("... stage 2 ...");
#endif

  winTVChannelWidget *winTVChannel= new winTVChannelWidget(NULL,"winTVChannel",rc);
  //winTVChannel->show();

  winTVVClipWidget *winTVVClip= new winTVVClipWidget(NULL,"winTVVClip",rc);

#ifdef myDEBUG
  debug("... stage 3 ...");
#endif

  winTVMainWidget *winTVMain = new winTVMainWidget(winTVApplication.argc(),winTVApplication.argv(),NULL,"winTVMain",rc);
  winTVApplication.setFilterObject(winTVMain->winTVScreenPtr());
  winTVApplication.setMainWidget(winTVMain);
 
  // connect winTVMain and winTVChannel
  QObject::connect(winTVMain,SIGNAL(channelChangedSIGNAL(channel *)),
		   winTVChannel,SLOT(activeChannelChangedSLOT(channel *)));
  QObject::connect(winTVMain,SIGNAL(newChannelListSIGNAL(channel *)),
		   winTVChannel,SLOT(newChannelListSLOT(channel *)));
  QObject::connect(winTVMain,SIGNAL(toggleChannelBoxSIGNAL()),
		   winTVChannel,SLOT(toggleVisibilitySLOT()));
  QObject::connect(winTVMain,SIGNAL(volumeChangedSIGNAL(int,int)),
		   winTVChannel,SLOT(volumeChangedSLOT(int,int)));
  QObject::connect(winTVMain,SIGNAL(toggleSplitSlidersSIGNAL()),
		   winTVChannel,SLOT(toggleSplitSlidersSLOT()));

  // connect winTVChannel and winTVMain
  QObject::connect(winTVChannel,SIGNAL(channelSelectedSIGNAL(channel *)),
		   winTVMain,SLOT(channelChangedSLOT(channel *)));
  QObject::connect(winTVChannel,SIGNAL(volumeChangedSIGNAL(int, int)),
		   winTVMain,SLOT(volumeSLOT(int, int)));
  /*
  QObject::connect(winTVChannel,SIGNAL(toggleSplitSlidersSIGNAL()),
		   winTVMain,SLOT(splitSlidersSLOT()));
		   */
  QObject::connect(winTVChannel,SIGNAL(addChannelSIGNAL()),
		   winTVMain,SLOT(addChannelSLOT()));
  QObject::connect(winTVChannel,SIGNAL(editChannelSIGNAL()),
		   winTVMain,SLOT(editChannelSLOT()));
  QObject::connect(winTVChannel,SIGNAL(deleteChannelSIGNAL()),
		   winTVMain,SLOT(deleteChannelSLOT()));

  // connect winTVMain and winTVVClip
  QObject::connect(winTVMain,SIGNAL(toggleVClipSIGNAL()),
		   winTVVClip,SLOT(toggleVClipSLOT()));
  QObject::connect(winTVMain,SIGNAL(frameVClipSIGNAL(int)),
		   winTVVClip,SLOT(frameVClipSLOT(int)));
  QObject::connect(winTVMain,SIGNAL(framelostVClipSIGNAL(int)),
		   winTVVClip,SLOT(framelostVClipSLOT(int)));
  QObject::connect(winTVMain,SIGNAL(statVideoClipFrameSIGNAL(double)),
		   winTVVClip,SLOT(statVideoClipFrameSLOT(double)));

  // connect winTVVClip winTVMain
  QObject::connect(winTVVClip,SIGNAL(startVClipSIGNAL(struct MOVIE_PARAMS *)),
		   winTVMain,SLOT(startVClipSLOT(struct MOVIE_PARAMS *)));
  QObject::connect(winTVVClip,SIGNAL(endVClipSIGNAL()),
		   winTVMain,SLOT(endVClipSLOT()));
  

  winTVMain->emitSetupSignals();
  winTVChannel->setPopups(winTVMain->getAudioMenu(),winTVMain->getChannelMenu());

  winTVMain->show();

#ifdef myDEBUG
  debug("... runing");
#endif

  // main loop
  winTVApplication.exec();

  // the end
#ifdef myDEBUG
  debug("going down...");
#endif
  //winTVMain->endAll();

#ifdef myDEBUG
  debug("... screen ...");
#endif

#ifdef withTVscreen
  // kill winTVscreen
  //winTVMain->winTVscreenPtr()->destroyTV();
#endif

#ifdef myDEBUG
  debug("... saving rc ...");
#endif

  rc->save(); 

#ifdef myDEBUG
  debug(" ... end");
#endif

}
//<-

//-> moc's
#include "wintv.moc"
//#include "winTVmainWidgetData.moc"
//<-

// local Variables: ***
// folded-file: t ***
// End: *** 

// in folding: C-c x
// leave fold: C-c y







