/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <wintv.h>

#include <wintvchannel.h>

#include <kapp.h>
#include <icondata.h>
#include <klocale.h>

#define BID_ADD     0
#define BID_EDIT    1
#define BID_DELETE  2
#define BID_SL      3
#define BID_SC      4 

#ifdef myDEBUG
#define chanDEBUG
#endif
/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/


winTVChannelWidget::winTVChannelWidget(QWidget* parent, const char* name, 
				       rcfile *_rc):  KTopLevelWidget( name ),rc(_rc)
{
#ifdef chanDEBUG
  debug("channel: begin of init");
#endif

  setCaption((const char*)(QString("channel box")));
  // set icon
  QPixmap *pm=new QPixmap(xwintv_xpm);
  setIcon(*pm);
  setIconText("kwintv - channel box");

  // toolbar
  toolbar = new KToolBar(this,"kwintv -- ToolBar --",40);
  toolbar->setBarPos(KToolBar::Top);
  addToolBar(toolbar);

  toolbar->insertButton(QPixmap(ch_add_xpm),BID_ADD,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("add new channel"));
  toolbar->insertButton(QPixmap(ch_edit_xpm),BID_EDIT,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("edit selected channel"));
  toolbar->insertButton(QPixmap(ch_delete_xpm),BID_DELETE,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("delete selected channel"));
  toolbar->insertSeparator();

  toolbar->insertButton(QPixmap(cc_xpm),BID_SL,SIGNAL(toggled(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("split/unsplit sliders"));
  toolbar->setToggle(BID_SL);
  toolbar->insertSeparator();

  toolbar->insertButton(QPixmap(cc_xpm),BID_SC,SIGNAL(toggled(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("hide channel box"));
  toolbar->setToggle(BID_SC);


  channelbox= new winWidget(this,"channel box and slider",rc->sliders_att_startup);
  rc->sliders_att=rc->sliders_att_startup;
  setView(channelbox);

  connect(channelbox,SIGNAL(highlighted(int)),this,SLOT(channelSelectedEmitSLOT(int)));
  connect(channelbox,SIGNAL(valueChanged(int,int)),this,SLOT(volumeChangedEmitSLOT(int, int)));

  setMinimumSize( 180,100 );
  resize( 200,200 );
}

/* ------------------------------------------------------------------------------*/

winTVChannelWidget::~winTVChannelWidget()
{

}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::newChannelListSLOT(channel *clist)
{
#ifdef chanDEBUG
  debug("channel: got new channel list");
#endif
  if (!clist) return;
  inserting=TRUE;
  channel *c=clist;
  clisthead=clist;
  channelbox->clear();
#ifdef chanDEBUG
  debug("channel: channel list inserting");
#endif
  c=c->next;
  while (c) {
    channelbox->insertItem(c->name);
    c=c->next;
  }
  inserting=FALSE;
#ifdef chanDEBUG
  debug("channel: channel list insertion done.");
#endif
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::activeChannelChangedSLOT(channel *c)
{
  aktive_channel=c->number - 1;
  channelbox->setCurrentItem(c->number - 1);
  channelbox->centerCurrentItem();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::volumeChangedSLOT(int vl,int vr)
{
  channelbox->setValue(vl,vr);
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::toggleSplitSlidersSLOT()
{
  rc->sliders_att=!rc->sliders_att;
  channelbox->splitSliderSLOT(rc->sliders_att);
  emit toggleSplitSlidersSIGNAL();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::toggleVisibilitySLOT()
{
  if (isVisible()) hide();
  else show();
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::volumeChangedEmitSLOT(int vl, int vr)
{
  emit volumeChangedSIGNAL(vl,vr);
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::channelSelectedEmitSLOT(int index)
{
  if (inserting) return;
  if (index != aktive_channel) {
    aktive_channel=index;
    index++;
    channel *head=clisthead->next;
    while ((head) && (head->number != index))
      head=head->next;
    
#ifdef chanDEBUG
    debug("channel: selected %s, %d, %d",head->name,head->number,index);
#endif

    emit channelSelectedSIGNAL(head);
  }
}

/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::buttonsSLOT(int button)
{
  switch (button) {
  case BID_SC:
    toggleVisibilitySLOT();
    break;
  case BID_SL:
    toggleSplitSlidersSLOT();
    break;
  case BID_ADD:
    emit addChannelSIGNAL();
    break;
  case BID_EDIT:
    emit editChannelSIGNAL();
    break;
  case BID_DELETE:
    emit deleteChannelSIGNAL();
    break;
  default:
    // this should never happen...
    fatal(klocale->translate("main: nonexisting button number"));
  }
}


/* ------------------------------------------------------------------------------*/

void winTVChannelWidget::setPopups(QPopupMenu *sp,QPopupMenu *cp)
{
 channelbox->setSPopup(sp);
 channelbox->setCPopup(cp);
}

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

#define Inherited QWidget

winWidget::winWidget(QWidget* parent,
		   const char* name,
		   int _sliderAtt)
  :sliderAtt(_sliderAtt),
   Inherited( parent, name, 503808 )
{
#ifdef chanDEBUG
  debug("win: init start");
#endif
       
  dlgedit_ListBox = new winListBox( this, "ListBox" );
  dlgedit_ListBox->setGeometry( 5, 5, 140, 148 );
  dlgedit_ListBox->setMinimumSize( 10, 10 );
  dlgedit_ListBox->setMaximumSize( 32767, 32767 );
  {
    QColorGroup normal( QColor( QRgb(0) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(32896) ), QColor( QRgb(16777215) ) );
    QColorGroup disabled( QColor( QRgb(8421504) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(8421504) ), QColor( QRgb(12632256) ) );
    QColorGroup active( QColor( QRgb(8388608) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(32896) ), QColor( QRgb(16777215) ) );
    QPalette palette( normal, disabled, active );
    dlgedit_ListBox->setPalette( palette );
  }
  {
    QFont font( "helvetica", 14, 20, 0 ); // 63
    font.setStyleHint( (QFont::StyleHint)0 );
    font.setCharSet( (QFont::CharSet)0 );
    dlgedit_ListBox->setFont( font );
  }
  dlgedit_ListBox->setFrameStyle( 51 );
  dlgedit_ListBox->setLineWidth( 2 );
  dlgedit_ListBox->setMultiSelection( FALSE );

  dlgedit_Slider_left = new winSlider( this, "Slider_left" );
  dlgedit_Slider_left->setGeometry( 155, 5, 20, 150 );
  dlgedit_Slider_left->setMinimumSize( 10, 10 );
  dlgedit_Slider_left->setMaximumSize( 32767, 32767 );
  dlgedit_Slider_left->setRange( 0, MIXER_MAX_VALUE );
  dlgedit_Slider_left->setSteps( 1, 10 );
  dlgedit_Slider_left->setValue( MIXER_MAX_VALUE );
  dlgedit_Slider_left->setTracking( TRUE );
  dlgedit_Slider_left->setTickmarks( QSlider::Left );
  dlgedit_Slider_left->setTickInterval( 0 );

  dlgedit_Slider_right = new winSlider( this, "Slider_right" );
  dlgedit_Slider_right->setGeometry( 175, 5, 20, 150 );
  dlgedit_Slider_right->setMinimumSize( 10, 10 );
  dlgedit_Slider_right->setMaximumSize( 32767, 32767 );
  dlgedit_Slider_right->setRange( 0, MIXER_MAX_VALUE );
  dlgedit_Slider_right->setSteps( 1, 10 );
  dlgedit_Slider_right->setValue( MIXER_MAX_VALUE );
  dlgedit_Slider_right->setTracking( TRUE );
  dlgedit_Slider_right->setTickmarks( QSlider::Right );
  dlgedit_Slider_right->setTickInterval( 0 );

  QToolTip::add(dlgedit_Slider_right,"set volume right");
  QToolTip::add(dlgedit_Slider_left,"set volume left");
  QToolTip::add(dlgedit_ListBox,"channels");

#ifdef chanDEBUG
  debug("win: init done, connecting");
#endif

  connect(dlgedit_Slider_right,SIGNAL(valueChanged(int)),this,SLOT(rightSliderSLOT(int)));
  connect(dlgedit_Slider_left,SIGNAL(valueChanged(int)),this,SLOT(leftSliderSLOT(int)));  
  connect(dlgedit_ListBox,SIGNAL(highlighted(int)),this,SLOT(highlightedSLOT(int)));

  if (sliderAtt) {
    dlgedit_Slider_right->hide();
    dlgedit_Slider_left->setTickmarks( QSlider::Both );
  }

  volume_left=0;
  volume_right=0;
}

// =============================================================================== 

void winWidget::setValue(int vl, int vr)
{
#ifdef chanDEBUG
  debug("win: setValue %d, %d",vl,vr);
#endif
  if ((vl != volume_left) || (vr != volume_right)) {
    volume_left=vl;
    volume_right=vr;
    if (sliderAtt) volume_right=volume_left;
    dlgedit_Slider_left->setValue(MIXER_MAX_VALUE-volume_left);
    dlgedit_Slider_right->setValue(MIXER_MAX_VALUE-volume_right);
  }
}

// =============================================================================== 

void winWidget::leftSliderSLOT(int value)
{
#ifdef chanDEBUG
  //debug("volumeSLOT %d",value);
#endif
  if (volume_left != MIXER_MAX_VALUE-value) {
    volume_left=MIXER_MAX_VALUE-value;
    if (sliderAtt && (volume_left != volume_right)) 
      dlgedit_Slider_right->setValue(MIXER_MAX_VALUE-volume_left);

    dlgedit_Slider_left->setValue(MIXER_MAX_VALUE-volume_left);
    
    emit valueChanged(volume_left,volume_right);
  }
}

// =============================================================================== 

void winWidget::rightSliderSLOT(int value)
{
#ifdef chanDEBUG
  //debug("volumeSLOT %d",value);
#endif
  if (volume_right != MIXER_MAX_VALUE-value) {
    volume_right=MIXER_MAX_VALUE-value;
    if (sliderAtt && (volume_left != volume_right)) 
      dlgedit_Slider_left->setValue(MIXER_MAX_VALUE-volume_right);
    
    dlgedit_Slider_right->setValue(MIXER_MAX_VALUE-volume_right);
    
    emit valueChanged(volume_left,volume_right);
  }
}

// =============================================================================== 

void winWidget::splitSliderSLOT(int split)
{
#ifdef chanDEBUG
  debug("win: splitSliderSLOT %s",split?"unsplip":"split");
#endif

  sliderAtt=split;

  if ( sliderAtt && (volume_left != volume_right) ) {
    dlgedit_Slider_right->setValue(MIXER_MAX_VALUE-volume_left);
  }

  if ( sliderAtt ) {
    dlgedit_Slider_right->hide();
    dlgedit_Slider_left->setTickmarks( QSlider::Both );
  } else {
    dlgedit_Slider_right->show();
    dlgedit_Slider_left->setTickmarks( QSlider::Left );
  }

  resizeEvent(NULL);

}

// =============================================================================== 

void winWidget::highlightedSLOT(int id)
{
#ifdef chanDEBUG
  debug("win:  highlightedSLOT %d",id);
#endif

  emit highlighted(id);
}

// =============================================================================== 

void winWidget::insertItem ( const char * text, int index)  
{
#ifdef chanDEBUG
  debug("win:  insertItem %s %d",text,index);
#endif
  dlgedit_ListBox->insertItem ( text, index );
}


// =============================================================================== 

void winWidget::removeItem ( int index ) 
{
  dlgedit_ListBox->removeItem ( index ); 
}

// =============================================================================== 

void winWidget::changeItem ( const char * text, int index ) 
{
  dlgedit_ListBox->changeItem ( text, index );
}

// =============================================================================== 

void winWidget::setCurrentItem ( int index )  
{
#ifdef chanDEBUG
  debug("win: setCurrentItem %d",index);
#endif
  dlgedit_ListBox->setCurrentItem ( index );  
}

// =============================================================================== 

void winWidget::centerCurrentItem ()  
{
  dlgedit_ListBox->centerCurrentItem ();
}

// =============================================================================== 

void winWidget::clear()  
{
  dlgedit_ListBox->clear();
}


// =============================================================================== 

void winWidget::resizeEvent( QResizeEvent * re)
{
#ifdef chanDEBUG
  debug("win: resizeEvent");
#endif

  if (!sliderAtt) {
    int sw=width()/5;
    dlgedit_ListBox->setGeometry( 5, 5, width()-sw-5, height()-10 );
    dlgedit_Slider_left->setGeometry( width()-sw+5, 5, sw/2-5, height()-10 );
    dlgedit_Slider_right->setGeometry( width()-sw+(sw/2), 5, sw/2-5, height()-10 );
  } else {
    int sw=width()/6;
    dlgedit_ListBox->setGeometry( 5, 5, width()-sw-10, height()-10 );
    dlgedit_Slider_left->setGeometry( width()-sw+5, 5, sw-10, height()-10 );
  }
}

// =============================================================================== 
// =============================================================================== 
// =============================================================================== 

void winSlider::mouseReleaseEvent ( QMouseEvent * me)
{
 if (me->button() == RightButton) {
#ifdef chanDEBUG
   debug("winSlider: mouseReleaseEvent");
#endif
   if (sPopup == 0) warning("winSlider: no popup");
   if (sPopup != 0) sPopup->popup(mapToGlobal(me->pos()));
  }
}

// =============================================================================== 

void winListBox::mouseReleaseEvent ( QMouseEvent * me)
{
  if (me->button() == RightButton) {
#ifdef chanDEBUG
    debug("winListBox: mouseReleaseEvent");
#endif
    if (cPopup == 0) warning("winListBox: no popup");
    if (cPopup != 0) cPopup->popup(mapToGlobal(me->pos()));
  }
}

// =============================================================================== 
// =============================================================================== 
// =============================================================================== 
// never change the next line!
#include "wintvchannel.moc"

// end of file


