/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef lswidget_included
#define lswidget_included

#include <wintv.h>

#include <qapp.h>
#include <qwidget.h>
#include <qlistbox.h>
#include <qtooltip.h>

#include <kpopmenu.h>
#include <qpopmenu.h>
#include <kslider.h>
#include <qslider.h> // use KSlider in later versions of KDE
#include <ktopwidget.h>

#include <rcfile.h>
#include <channel.h>

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winSlider : public QSlider
{
public:
  winSlider ( QWidget * parent=0, const char * name=0 ):QSlider ( parent, name ) {}   
  // ~winSlider();

  void setSPopup(QPopupMenu *sp) { sPopup=sp; }

protected:
  /* ------------------------------------------------------------------------------*/
  void mouseReleaseEvent ( QMouseEvent * me);

private:
  QPopupMenu *sPopup;
};

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winListBox : public QListBox
{
public:
  winListBox ( QWidget * parent=0, const char * name=0, WFlags f=0 )
    :QListBox ( parent,name,f ) {}
  // ~winListBox();

  void setCPopup(QPopupMenu *sp) { cPopup=sp; }

protected:
  /* ------------------------------------------------------------------------------*/
  void mouseReleaseEvent ( QMouseEvent * me);

private:
  QPopupMenu *cPopup;
};

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winWidget : public QWidget
{
  Q_OBJECT

public:
  winWidget(QWidget* parent = NULL,
	   const char* name = NULL,
	   int _sliderAtt=1);
  //~winWidget();

  void setSPopup(QPopupMenu *sp) {sPopup=sp;dlgedit_Slider_right->setSPopup(sPopup);dlgedit_Slider_left->setSPopup(sPopup);} 
  void setCPopup(QPopupMenu *cp) { cPopup=cp;dlgedit_ListBox->setCPopup(cPopup); } 

public slots:
  void setValue ( int vl, int vr );
  void splitSliderSLOT(int split);
  void insertItem ( const char * text, int index=-1);  
  void removeItem ( int index ); 
  void changeItem ( const char * text, int index ); 
  void setCurrentItem ( int index );  
  void centerCurrentItem ();  
  void clear();

protected slots:
  void leftSliderSLOT(int value);
  void rightSliderSLOT(int value);
  void highlightedSLOT(int id);

protected:
  /* ------------------------------------------------------------------------------*/
  //void mouseReleaseEvent ( QMouseEvent * me);
  void resizeEvent( QResizeEvent * );

signals:
  void highlighted(int id);
  void valueChanged(int vl, int vr);
  void muteChanged(int mute);

private:
  winSlider* dlgedit_Slider_right;
  winSlider* dlgedit_Slider_left;

  winListBox* dlgedit_ListBox;
  QPopupMenu *sPopup,*cPopup;

  int sliderAtt;
  int volume_right,volume_left;
};

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

class winTVChannelWidget : public KTopLevelWidget
{
  Q_OBJECT

public:
  winTVChannelWidget(QWidget* parent = NULL,const char* name = NULL, rcfile *_rc=NULL );
  ~winTVChannelWidget();

  void setPopups(QPopupMenu *sp,QPopupMenu *cp);

public slots:
  void newChannelListSLOT(channel *clist);
  void activeChannelChangedSLOT(channel *c);
  void volumeChangedSLOT(int vl,int vr);
  void toggleSplitSlidersSLOT();
  void toggleVisibilitySLOT();

protected slots:
  void buttonsSLOT(int);
  void volumeChangedEmitSLOT(int, int);
  void channelSelectedEmitSLOT(int);

signals:
  void channelSelectedSIGNAL(channel *);
  void addChannelSIGNAL();
  void editChannelSIGNAL();
  void deleteChannelSIGNAL();
  void volumeChangedSIGNAL(int, int);
  void toggleSplitSlidersSIGNAL();

private:
  winWidget *channelbox;
  KToolBar *toolbar;

  rcfile *rc;

  channel *clisthead;

  int aktive_channel;
  bool inserting;
};

#endif





