/* 
    xwintv using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <unistd.h>

#include <qfiledlg.h>

#include <wintvmain.h>

//#include <qdir.h> 
#include <qfiledlg.h> 
#include <qkeycode.h>
#include <qmsgbox.h>

#include <kapp.h>
#include <klocale.h>

#include <icondata.h> // the icons 

#include <countries.h>

#undef HAVE_LIBXXF86VM
#ifdef HAVE_LIBXXF86VM
#include <X11/extensions/xf86vmode.h>
#endif

#define BID_UP      0
#define BID_DOWN    1
#define BID_CAP     2 
#define BID_SS      3
#define BID_FAR     4
#define BID_FSS     5
#define BID_VUP     6
#define BID_MUTE    7
#define BID_VDOWN   8
#define BID_SC      9
#define BID_EXIT   10
#define BID_FRAME  11

winTVMainWidget::winTVMainWidget ( int argc, char **argv, QWidget* parent,
			   const char* name, rcfile *_rc
			   ): rc(_rc), KTopLevelWidget( name )
{
#ifdef mainDEBUG
  debug("main: begin of init");
#endif

  setCaption((const char*)(QString("kwintv ")+QString(VERSION)));
  // set icon
  QPixmap *pm=new QPixmap(xwintv_xpm);
  setIcon(*pm);
  setIconText("kwintv");

  // toolbar
  toolbar = new KToolBar(this,"kwintv -- ToolBar --");
  toolbar->setBarPos(KToolBar::Top);
  toolbar->insertButton(QPixmap(ch_up_xpm),BID_UP,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("switch channel up"));
  toolbar->insertFrame(BID_FRAME,25); // well, well
  toolbar->insertButton( QPixmap(ch_down_xpm),BID_DOWN,SIGNAL(released(int)),this,
			 SLOT(buttonsSLOT(int)),true,klocale->translate("switch channel down"));
  toolbar->insertSeparator();

  toolbar->insertButton(QPixmap(uncapture_xpm),BID_CAP,SIGNAL(toggled(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("toggle capture"));
  toolbar->setToggle(BID_CAP);
  toolbar->insertButton(QPixmap(snap_xpm),BID_SS,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("screen shot"));
  toolbar->insertSeparator();

  toolbar->insertButton(QPixmap(aspect_xpm),BID_FAR,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("fix aspect ratio"));
  toolbar->insertButton(QPixmap(fullsize_xpm),BID_FSS,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("toggle full screen size"));
  toolbar->insertSeparator();

  toolbar->insertButton(QPixmap(vol_up_xpm),BID_VUP,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("volme up"));
  toolbar->insertButton(QPixmap(mute_xpm),BID_MUTE,SIGNAL(toggled(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("mute/unmute sound"));
  toolbar->setToggle(BID_MUTE);
  toolbar->insertButton(QPixmap(vol_down_xpm),BID_VDOWN,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("volme down"));
  toolbar->insertSeparator();

  toolbar->insertButton(QPixmap(cc_xpm),BID_SC,SIGNAL(toggled(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("show/hide channel box"));
  toolbar->setToggle(BID_SC);
  toolbar->insertButton(QPixmap(exit_xpm),BID_EXIT,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,klocale->translate("exit"));

  addToolBar(toolbar);

  // popups for menubar  
  QPopupMenu *impmenu=new QPopupMenu;
  impmenu->insertItem("x&awtv",this,SLOT(importXAWTVChannelsSLOT()));
  impmenu->insertItem("&xtvscreen",this,SLOT(importXTVSCREENChannelsSLOT()));

  QPopupMenu *optmenu=new QPopupMenu;
  optmenu->insertItem(klocale->translate("Hide Toolbar"),this,SLOT(toggleToolBarSLOT()));
  optmenu->insertItem(klocale->translate("Hide Mousecursor"),this,SLOT(naSLOT()));
  optmenu->insertSeparator();
  optmenu->insertItem(klocale->translate("General &Options"),this,SLOT(setupSLOT()));
  optmenu->insertItem(klocale->translate("&Save Options"),this,SLOT(saveRCSLOT()));
#ifdef HAVE_IRC
  // we dont have IRC support till now
  optmenu->insertSeparator();
  optmenu->insertItem(klocale->translate("&IR-Contorl"),this,SLOT(IRCoptionsSLOT()));
#endif

  QPopupMenu *filemenu=new QPopupMenu;
  filemenu->insertItem(klocale->translate("&Load"),this,SLOT(loadChannelsSLOT()));
  filemenu->insertItem(klocale->translate("&Save"),this,SLOT(saveChannelsSLOT()));
  filemenu->insertItem(klocale->translate("Save &As"),this,SLOT(saveChannelsAsSLOT()));
  filemenu->insertSeparator();
  filemenu->insertItem(klocale->translate("&Import"),impmenu);
  filemenu->insertSeparator();
  filemenu->insertItem(klocale->translate("E&xit"),qApp,SLOT(quit()));
	
  chmenu=new QPopupMenu;
  chmenu->insertItem(klocale->translate("&Show/Hide Box"),this,SLOT(toggleChannelBoxSLOT()));
  chmenu->insertSeparator();
  chmenu->insertItem(klocale->translate("&Add"),this,SLOT(addChannelSLOT()));
  chmenu->insertItem(klocale->translate("&Edit"),this,SLOT(editChannelSLOT()));
  chmenu->insertItem(klocale->translate("&Delete"),this,SLOT(deleteChannelSLOT()));

  //int audio=winTVScreen->getAudioMode();
  int audio=1;
  smenu=new QPopupMenu;
  smenu->setCheckable( TRUE );
  smid[0]=smenu->insertItem(klocale->translate("&Autodetection"),this,SLOT(audioAutoSLOT()));
  smenu->setItemChecked(smid[0], audio == 0); 
  smid[1]=smenu->insertItem(klocale->translate("&Mono"),this,SLOT(audioMonoSLOT()));
  smenu->setItemChecked(smid[1], audio == VIDEO_SOUND_MONO); 
  smid[2]=smenu->insertItem(klocale->translate("&Stereo"),this,SLOT(audioStereoSLOT()));
  smenu->setItemChecked(smid[2], audio == VIDEO_SOUND_STEREO); 
  smid[3]=smenu->insertItem(klocale->translate("Language &1"),this,SLOT(audioL1SLOT()));
  smenu->setItemChecked(smid[3], audio == 3); 
  smid[4]=smenu->insertItem( klocale->translate("Language &2"),this,SLOT(audioL2SLOT()));
  smenu->setItemChecked(smid[4], audio == 4); 
  smenu->insertSeparator();
  smid[5]=smenu->insertItem(klocale->translate("M&ute"),this,SLOT(soundSLOT()));
  smenu->setItemChecked(smid[5], !rc->sound_on); 
  smenu->insertSeparator();
  smid[6]=smenu->insertItem(klocale->translate("S&plit sliders"),this,SLOT(splitSlidersSLOT()));
  smenu->setItemChecked(smid[6], FALSE ); 

  // FIX ME
  /*
  if ( audio == VIDEO_SOUND_MONO ) {
  smenu->setItemEnabled (smid[2],FALSE);
  smenu->setItemEnabled (smid[3],FALSE);
  smenu->setItemEnabled (smid[4],FALSE);
  }
  */

  tvmenu=new QPopupMenu;
  tvmenu->insertItem(klocale->translate("&Capture on/off"),this,SLOT(captureToggleSLOT()));
  tvmenu->insertItem(klocale->translate("&Full Screen"),this,SLOT(fullScreenSLOT()));
  tvmenu->insertItem(klocale->translate("Fix &Aspect Ratio"),this,SLOT(adjustAspectRatioSLOT()));
  tvmenu->insertSeparator();
  tvmenu->insertItem(klocale->translate("&Snap Shot"),this,SLOT(screenShotSLOT()));
  tvmenu->insertItem(klocale->translate("&Video Clip"),this,SLOT(videoClipSLOT()));
  tvmenu->insertSeparator();
  tvmenu->insertItem(klocale->translate("C&hannel"),chmenu);
  tvmenu->insertSeparator();
  tvmenu->insertItem(klocale->translate("A&udio"),smenu);

  // help menu
  QString about(klocale->translate(
		     "       kwintv is a X11 applications for the\n"
		     "           bttv - BT848 frame grabber\n\n"
		     "         Copyright 1998 by Moritz Wenk\n\n"
		     "This Program is distributed under the terms of the\n"
		     "             GNU General Public License\n\n"
		     "Send any comments, gifts, homages, abusings etc. to:\n" 
		     "              wenk@mathematik.uni-kl.de\n\n"
		     "             For new versions, look at:\n"
		     "  http://www.mathematik.uni-kl.de/~wenk/xwintv.html"));

  QPopupMenu *helpmenu=KApplication::getKApplication()->getHelpMenu(TRUE,about); 
  helpmenu->insertSeparator();
  helpmenu->insertItem(klocale->translate("&Properties..."),this,SLOT(propertiesSLOT()));

  // menu bar  
  menubar = new KMenuBar( this, "kwintv -- MenuBar --" );
  menubar->insertItem(klocale->translate("&File"),filemenu);
  menubar->insertItem(klocale->translate("&TeleVision"),tvmenu);
#ifdef HAVE_VTX
  // we dont have vtx till now...
  //menubar->insertItem(klocale->translate("&VideoText"),this,SLOT(vtxSLOT()));
#endif
  menubar->insertItem(klocale->translate("&Options"),optmenu);
  menubar->insertSeparator();
  menubar->insertItem(klocale->translate("&Help"),helpmenu);
  setMenu(menubar);

  // the lcd with the actual channel number
  lcd_channel = new QLCDNumber(toolbar, "LCDNumber_1" );
  lcd_channel->setMinimumSize( 10, 10 );
  lcd_channel->setMaximumSize( 32767, 32767 );

  {
    QColorGroup normal( QColor( QRgb(9355867) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ) );
    QColorGroup disabled( QColor( QRgb(8421504) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(8421504) ), QColor( QRgb(12632256) ) );
    QColorGroup active( QColor( QRgb(0) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ) );
    QPalette palette( normal, disabled, active );
    lcd_channel->setPalette( palette );
  }

  lcd_channel->setSmallDecimalPoint( FALSE );
  lcd_channel->setNumDigits( 2 );
  lcd_channel->setMode( QLCDNumber::DEC );
  lcd_channel->setSegmentStyle( QLCDNumber::Flat );
  lcd_channel->setFrameStyle( QFrame::NoFrame );
  lcd_channel->setGeometry( toolbar->getFrame(BID_FRAME)->x(),toolbar->getFrame(BID_FRAME)->y(),
			    toolbar->height() - 1 , toolbar->height() - 1);
  lcd_channel->raise();

  //QColor bkgc(0,0,0);
  //toolbar->getFrame(BID_FRAME)->setBackgroundColor(bkgc);

  /////////////////////////////////////////////////////////////////////////
  // channel info
  channelLCD = new lcdData( NULL , "channelLCD" );
  channelLCD->hide();
  //channelLCDpos=UPPER_RIGHT;
  channelLCDpos=NONE; // deactivated

  //infoTimer = new QTimer(this);
  //connect(infoTimer,SIGNAL(timeout()),this,SLOT(unShowChannelInfoSLOT()));

  /////////////////////////////////////////////////////////////////////////////
  // load contry-file, see countries.h for valid internal countries
  //loadcountry(rc->country_id,(const char*)rc->country_name);

  ///////////////////////////////////////////////////////////////////////
  // load channel-file
  selchnl=0;
  loadChannels((const char*)rc->channelListName);

  /////////////////////////////////////////////////////////////////////////////
  // init sound mixer
  mix= new mixer((const char *)rc->mixerDeviceName,rc->mixerDevice);

  if (rc->use_master_volume) {
    mix->get_vol(&rc->volume_left,LEFT);
    mix->get_vol(&rc->volume_right,RIGHT);
  }

  /////////////////////////////////////////////////////////////////////////
  // set up winTVScreen widget

  // first parse some command line options
  int i,bpp=0;
  int dual=0;
  for ( i=0; i < argc; i++ ) {
    if ( strcmp(argv[i],"-bpp") == 0 ) { 
      if ( argc >= i+1 ) {
	i++;
	bpp=atoi(argv[i]);
	switch (bpp) {
	case 8:
	case 15:
	case 16:
	case 24:
	case 32:
	  break;
	default:
	  fatal(klocale->translate("main: incorrect value %d given for switch -bpp. Valid values are 8,15,16,24,32."),bpp);
	}
      } else fatal(klocale->translate("main: no parameter given for switch -bpp!"));
    }
    if ( (strcmp(argv[i],"-dev") == 0) || (strcmp(argv[i],"-device") == 0) ) {
      if ( argc >= i+1 ) {
	i++; 
	rc->bttvDeviceName=argv[i];
      }
    }
    if ( (strcmp(argv[i],"-cap") == 0) || (strcmp(argv[i],"-capture") == 0) ) {
      rc->capture=1;
    }
    if ( (strcmp(argv[i],"-dual") == 0) ) {
      dual=1;
    }
    if ( (strcmp(argv[i],"-h") == 0) || (strcmp(argv[i],"-help") == 0) ) {
      printf(klocale->translate("This is kwintv " VERSION "\n"
      "Aviable command line switches: \n"
      "\t -device DEVICENAME         Set bttv device to DEVICENAME (default is /dev/bttv).\n"
      "\t -bpp [8|15|16|24|32]       Force colour depth of given value.\n"
      "\t -capture                   Set capture on after startup. Default is off. USE WITH CARE!\n"
      "\t -dual                      Force dual mode for graphic cards with dual buffers.\n"
      "See README for details.\n\n"));
      qApp->exit();
      exit(0);
    }
  }

  // now set up ...
#ifdef withTVscreen
  winTVScreen = new winTVScreenWidget(this,
				      "winTVScreen",
				      0,//rc->capture,
				      rc->automouse,
				      (char *)(const char *) rc->bttvDeviceName,
				      bpp,dual);
	
  // set winTVScreen values
  if (rc->screensaver) winTVScreen->toggleScreenSaverSLOT();

  //	winTVScreen->changeMuteSLOT(); 
  winTVScreen->setFocusPolicy ( QWidget::StrongFocus );
  winTVScreen->setFocus();
  winTVScreen->setTVPopup(tvmenu);
  setView(winTVScreen);
#endif

  /////////////////////////////////////////////////////////////////////////////
  // set up ctab dialogs
  tab= new ctab(NULL,"ctab");
  options= new coptions(tab,"options");
  selector= new cselector(tab,"selector",rc,winTVScreen->getCapCap(),
			  winTVScreen->getCapChannel(),winTVScreen->getCapTuner(),winTVScreen->getCapAudio());
  //  freq= new cfreq(tab,"freq");
  // cfreq for later version

  tab->addTab(selector,"&Channel");
  tab->addTab(options,"&Options");
  //tab->addTab(freq,"&Frequency");

  tab->setCancelButton();
  tab->setOkButton();
  tab->setDefaultButton();

  /////////////////////////////////////////////////////////////////////////////
  // set up otab dialogs
  mtab= new otab(NULL,"otab");
  mixdev= new omixdev(mtab,"mixdev",rc,mix);

  mtab->addTab(mixdev,"&Mixer");

  mtab->setOkButton();
  //mtab->setCancelButton();
  //mtab->setDefaultButton();

  /////////////////////////////////////////////////////////////////////////
  // connect everything
  connect(winTVScreen,SIGNAL(keyPressedSIGNAL(int)),this,SLOT(keyPressedSLOT(int)));
  connect(winTVScreen,SIGNAL(autoDetectAudioModeSIGNAL(int)),this,SLOT(audioModeChangedSLOT(int)));
  connect(winTVScreen,SIGNAL(wroteVideoClipFrameSIGNAL(int)),this,SLOT(frameVClipSLOT(int)));
  connect(winTVScreen,SIGNAL(lostVideoClipFrameSIGNAL(int)),this,SLOT(framelostVClipSLOT(int)));
  connect(winTVScreen,SIGNAL(statVideoClipFrameSIGNAL(double)),this,SLOT(statVideoClipFrameSLOT(double)));

  connect(this,SIGNAL(channelChangedSIGNAL(channel *)),this,SLOT(channelChangedSLOT(channel *)));

  // connect capture changing
  connect(this,SIGNAL(captureToggleSIGNAL(int)),winTVScreen,SLOT(changeCaptureSLOT(int)));

  // connect audio
  connect(this,SIGNAL(audioModeChangedSIGNAL(int)),winTVScreen,SLOT(changeAudioModeSLOT(int)));
  connect(this,SIGNAL(audioMuteToggledSIGNAL()),winTVScreen,SLOT(toggleAudioMuteSLOT()));
  if (rc->use_tv_volume) 
    connect(this,SIGNAL(volumeChangedSIGNAL(int,int)),winTVScreen,SLOT(changeAudioVolumeSLOT(int,int)));

  // connect brightness, contrast, colour, hue
  connect(options,SIGNAL(newS1ValueSIGNAL(int)),winTVScreen,SLOT(changeBrightnessSLOT(int)));
  connect(options,SIGNAL(newS2ValueSIGNAL(int)),winTVScreen,SLOT(changeContrastSLOT(int)));
  connect(options,SIGNAL(newS3ValueSIGNAL(int)),winTVScreen,SLOT(changeColorSLOT(int)));
  connect(options,SIGNAL(newS4ValueSIGNAL(int)),winTVScreen,SLOT(changeHueSLOT(int)));

  // connect tune, input, channel 
  connect(selector,SIGNAL(frequencyChangedSIGNAL(int)),winTVScreen,SLOT(changeFrequencySLOT(int)));
  connect(selector,SIGNAL(inputChangedSIGNAL(int)),winTVScreen,SLOT(changeSourceSLOT(int)));
  connect(selector,SIGNAL(normChangedSIGNAL(int)),winTVScreen,SLOT(changeNormSLOT(int)));

  // connect ctab dialog apply, cancel, default
  connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyTabSLOT())); 
  connect(tab,SIGNAL(cancelButtonPressed()),this,SLOT(cancelTabSLOT())); 
  connect(tab,SIGNAL(defaultButtonPressed()),this,SLOT(defaultTabSLOT()));

  // connect cmtab dialog apply, cancel, default
  connect(mtab,SIGNAL(applyButtonPressed()),this,SLOT(applyMtabSLOT())); 
  //connect(mtab,SIGNAL(cancelButtonPressed()),this,SLOT(cancelMtabSLOT())); 
  //connect(mtab,SIGNAL(defaultButtonPressed()),this,SLOT(defaultMtabSLOT()));

  // mute/unmute sound
  if (!rc->sound_on) {
    // mute
    mix->set_vol(0,0);
    emit audioMuteToggledSIGNAL();
  } else {
    mix->set_vol(rc->volume_left,rc->volume_right);
    //volumeSLOT(rc->volume_left,rc->volume_right);
  }

  // some default settings
  fullscreen=FALSE;
  capture=rc->capture;
  fixed=FALSE;

  // FIX ME
  rc->aspectRatio=ASPECT_4_3;
  rc->autoar=0;

  // resize etc.
  resize( 384,288+toolbar->height()+menubar->height() );
  setMinimumSize( 384/2,288/2 );
#ifdef mainDEBUG
  debug("main: end of init");
#endif
}

// =================================================================================================

winTVMainWidget::~winTVMainWidget()
{
  if (rc->autosave) saveChannels((const char *)rc->channelListName);
}

// =================================================================================================

void winTVMainWidget::audioModeChangedSLOT(int mode)
{
  if ( !selchnl ) return;
  if (mode & AUDIOMODE_AUTODETECT) {
    setCaption((const char*)(QString("kwintv - ")+QString(selchnl->name)
			     +QString(" - (auto) ")
			     +QString(audiomode[mode ^ AUDIOMODE_AUTODETECT].str)));
  } else {
    setCaption((const char*)(QString("kwintv - ")+QString(selchnl->name)
			     +QString(" - ")+QString(audiomode[mode].str)));
  }
  showChannelInfoSLOT(selchnl);
}

// =================================================================================================

void winTVMainWidget::channelChangedSLOT(channel *ch)
{
  selchnl=ch;
  setCaption((const char*)(QString("kwintv - ")+QString(ch->name)
			   +QString(" - ")+QString(audiomode[winTVScreen->getAudioMode()].str)));
  lcd_channel->display(ch->number);
  showChannelInfoSLOT(ch);
  winTVScreen->changeChannelValuesSLOT(ch);
}

// =================================================================================================

void winTVMainWidget::keyPressedSLOT(int key)
{
#ifdef mainDEBUG
  debug(klocale->translate("main:  keyPressedSLOT %d"),key);
#endif
  switch (key) {
  case Key_Minus:
    channelDownSLOT();
    break;
  case Key_Plus:
    channelUpSLOT();
    break;
  case Key_C:
    captureToggleSLOT();
    break;
  case Key_B:
    emit toggleChannelBoxSIGNAL();
    break;
  case Key_S:
    screenShotSLOT();
    break;
  case Key_A:
    adjustAspectRatioSLOT();
    break;
  case Key_P:
  case Key_F:
    fullScreenSLOT();
    break;
  case Key_M:
    winTVScreen->toggleMouseCursorSLOT();
    break;
#ifdef mainDEBUG
  default:
    warning(klocale->translate("main: unsupported key: %d"), key);
#endif
  }
}

// =================================================================================================

void winTVMainWidget::fullScreenSLOT()
{
  fullscreen=!fullscreen;

#ifdef mainDEBUG
  debug(klocale->translate("main: set screen size to %s"),fullscreen?"full":"normal");
#endif
  
  //winTVScreen->changeCaptureSLOT(0);
  if (!fullscreen) {
    // old size with frame etc.
    toolbar->enable(KToolBar::Show);
    menubar->show();
    QPoint p(oldPos.x(),oldPos.y());
    setGeometry(oldPos);
    
    winTVScreen->setFocusPolicy ( QWidget::StrongFocus );
    winTVScreen->setFocus();
  } else {
    // full screen size
    //QPoint p(-20,-20);
    oldPos=geometry();
    toolbar->enable(KToolBar::Hide);
    menubar->hide();
#ifdef HAVE_LIBXXF86VM    
    int vp_x,vp_y;
    XF86VidModeGetViewPort(qt_xdisplay(),XDefaultScreen(qt_xdisplay()),&vp_x,&vp_y);
    setGeometry(vp_x,vp_y,width(),height());
#else
    QWidget *d = QApplication::desktop();
    setGeometry(-20,-20,d->width()+20,d->height()+20);
#endif HAVE_LIBXXF86VM
    winTVScreen->setFocusPolicy ( QWidget::StrongFocus );
    winTVScreen->setFocus();
  }
  
}
 
// =================================================================================================

void winTVMainWidget::adjustAspectRatioSLOT()
{
  int ar=rc->aspectRatio;

  if ((ar == ASPECT_NONE) || fixed) {
    fixed=FALSE;
    return;
  }

  fixed=TRUE;

  int xa,ya;
  int Xoffset=0,Yoffset=0;

  switch (ar)
    {
    case ASPECT_4_3:
      xa=4; ya=3;
      break;
      
    case ASPECT_16_9: 
      xa=16; ya=9;
      break;
    default:
      warning(klocale->translate("main: incorrect value for aspect ratio: %d"),ar);
      return;
    }

  if ((menubar->menuBarPos() != KMenuBar::Floating) && (menubar->isVisible())) {
    Yoffset+=menubar->height();
  }

  if ((toolbar->barPos() != KToolBar::Floating) && (toolbar->isVisible())) {
    if ((toolbar->barPos() == KToolBar::Left) || (toolbar->barPos() == KToolBar::Right))
      Xoffset+=toolbar->width();
    if((toolbar->barPos() == KToolBar::Top) || (toolbar->barPos() == KToolBar::Bottom))
      Yoffset+=toolbar->height();
  }
  
  // aktuelle Breite u Hoehe der tv screen
  QRect size=winTVScreen->geometry();

  if ((size.height() < (ya*(size.width()))/xa)) {
    size.setHeight((ya*(size.width()))/xa);
    resize(size.width()+Xoffset,size.height()+Yoffset);
  } else 
    if (size.width() < (xa*(size.height()))/ya) {
      size.setWidth((xa*(size.height()))/ya);
      resize(size.width()+Xoffset,size.height()+Yoffset);
    } else {
      fixed=FALSE;
    }

#ifdef mainDEBUG
  debug("main: adjusted aspect ratio");
#endif
}

// =================================================================================================

void winTVMainWidget::buttonsSLOT(int b)
{
#ifdef mainDEBUG
  //debug(klocale->translate("main: button %d klicked"),b);
#endif

  switch (b) {
  case BID_DOWN: // switch channel down
    channelDownSLOT();
    break;
  case BID_UP:   // switch channel up
    channelUpSLOT();
    break;
  case BID_CAP:  // toggle capture 
    captureToggleSLOT();
    break;
  case BID_SS:   // make a screen shot
    screenShotSLOT();
    break;
  case BID_SC:   // show window with channel list and volume control ???
    emit toggleChannelBoxSIGNAL();
    break;
  case BID_EXIT: // exit
    KApplication::getKApplication()->quit();
    break;
  case BID_FAR:  // adjust aspect ratio
    adjustAspectRatioSLOT();
    break;
  case BID_VUP:  // volume up
    volumeUpSLOT();
    break;
  case BID_MUTE: // toggle mute/unmute
    soundSLOT();
    break;
  case BID_VDOWN:// voluem down
    volumeDownSLOT();
    break;
  case BID_FSS:  // toggle full screen
    fullScreenSLOT();
    break;
  default:       // this should never happen...
    fatal("main: hardcore error 00004");
  }
}


// =============================================================================== 

void winTVMainWidget::toggleChannelBoxSLOT()
{
  emit toggleChannelBoxSIGNAL();
}

// =============================================================================== 

void winTVMainWidget::captureToggleSLOT()
{
  capture=!capture;
#ifdef withTVscreen
  emit captureToggleSIGNAL(capture);
#endif
}

// =============================================================================== 

void winTVMainWidget::screenShotSLOT()
{
#ifdef withTVscreen
  winTVScreen->snapShotSLOT(rc->fullSize);
#endif
}

// =============================================================================== 

void winTVMainWidget::videoClipSLOT()
{
  emit toggleVClipSIGNAL();
}

// =============================================================================== 

void winTVMainWidget::startVClipSLOT(struct MOVIE_PARAMS *par)
{
  winTVScreen->startVideoClipSLOT(par);
}

// =============================================================================== 

void winTVMainWidget::endVClipSLOT()
{
  winTVScreen->endVideoClipSLOT();
}

// =============================================================================== 

void winTVMainWidget::frameVClipSLOT(int c)
{
  emit frameVClipSIGNAL(c);
}

void winTVMainWidget::statVideoClipFrameSLOT(double d)
{
  emit statVideoClipFrameSIGNAL(d);
}

// =============================================================================== 

void winTVMainWidget::framelostVClipSLOT(int c)
{
  emit framelostVClipSIGNAL(c);
}

// =============================================================================== 

void winTVMainWidget::setupSLOT()
{
  if (!rc->sound_on) soundSLOT();
  mixdev->updateData();
  mtab->show();
}

// =============================================================================== 

void winTVMainWidget::audioAutoSLOT()
{
  emit audioModeChangedSIGNAL(0);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i], (winTVScreen->getAudioMode()+1) == (i+1));
}

// =============================================================================== 

void winTVMainWidget::audioMonoSLOT()
{
  emit audioModeChangedSIGNAL(VIDEO_SOUND_MONO);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i],winTVScreen->getAudioMode() == i);
}

// =============================================================================== 

void winTVMainWidget::audioStereoSLOT()
{
  emit audioModeChangedSIGNAL(VIDEO_SOUND_STEREO);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i],winTVScreen->getAudioMode() == i);
}

// =============================================================================== 

void winTVMainWidget::audioL1SLOT()
{
  emit audioModeChangedSIGNAL(VIDEO_SOUND_LANG1);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i],winTVScreen->getAudioMode() == i);
}

// =============================================================================== 

void winTVMainWidget::audioL2SLOT()
{
  emit audioModeChangedSIGNAL(VIDEO_SOUND_LANG2);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i],winTVScreen->getAudioMode() == i);
}

// =============================================================================== 

void winTVMainWidget::addChannelSLOT()
{
  //debug("main: att channel, channel count %d",selchnl->count());
  if (selchnl) {
    options->setDataSLOT(selchnl);
    selector->setDataSLOT(selchnl);
  } else {
    //options->setDataSLOT(selchnl);
    fatal("main: hardcore error 00003");
  }
  //if (!winTVScreen->getCaptureState()) captureToggleSLOT();

  wasEdit=FALSE;
  tab->setCaption("kwintv - add a new channel");
  tab->show();
}

// =============================================================================== 

void winTVMainWidget::editChannelSLOT()
{
  if (!selchnl->number) return;

  //if (!winTVScreen->getCaptureState()) captureToggleSLOT();

  options->setDataSLOT(selchnl);
  selector->setDataSLOT(selchnl);

  wasEdit=TRUE;
  tab->setCaption((const char *)(QString("kwintv - edit channel <")
				 +QString(selchnl->name)+QString(">")));
  tab->show();
}

// =============================================================================== 

void winTVMainWidget::deleteChannelSLOT()
{
  channel *newchannel;
  int number;

  if (!selchnl) return;

#ifdef mainDEBUG
  debug("deleting channel %d, %s",selchnl->number-1,selchnl->name);
#endif
  if (selchnl->prev == &channel_head) {
    if (selchnl->next == NULL) {
      newchannel=NULL;
    } else {
      newchannel=selchnl->next;
    }
  } else {
    newchannel=selchnl->prev;
  }

  number=selchnl->number-1;
  selchnl->~channel();
  free(selchnl);
  selchnl=newchannel;
    
  if (!selchnl) {
    newchannel= new channel(&channel_head,rc->freq,"<no channel>",
			    rc->col,rc->bri,rc->hue,rc->contr,rc->norm,rc->in);
    selchnl=newchannel;
#ifdef mainDEBUG
    debug("new nonexistent channel %d, %s",selchnl->number-1,selchnl->name);
#endif
  }

#ifdef mainDEBUG
  debug("new aktual channel %d, %s",selchnl->number-1,selchnl->name);
#endif
  emit newChannelListSIGNAL(&channel_head);
  selchnl=newchannel;
  emit channelChangedSIGNAL(selchnl);
}

// =============================================================================== 

void winTVMainWidget::cancelTabSLOT()
{
#ifdef withTVscreen
  emit channelChangedSIGNAL(selchnl);
#endif
}

// =============================================================================== 

void winTVMainWidget::defaultTabSLOT()
{
  if (selchnl) {
    options->setDataSLOT(selchnl);
    selector->setDataSLOT(selchnl);
  } else {
    //options->setDataSLOT(selchnl);
  }
  
  // call winTVscreen-Widget with original settings
#ifdef withTVscreen
  emit channelChangedSIGNAL(selchnl);
#endif
}

// =============================================================================== 

void winTVMainWidget::applyTabSLOT()
{
  if (wasEdit) {
    selchnl->startwiththis=selector->getData()->startwiththis;
    if (selchnl->startwiththis) {
      channel *n;
      n=&channel_head;
      while (n->next) {
	n=n->next;
	if (n != selchnl) {
	  n->startwiththis=0;
	}
      }
    }
    // copy new values into channel
    selchnl->frequency=selector->getData()->frequency;
    selchnl->input=selector->getData()->input;
    selchnl->norm=selector->getData()->norm;
    selchnl->brightness=options->getData()->brightness;
    selchnl->contrast=options->getData()->contrast;
    selchnl->color=options->getData()->color;
    selchnl->hue=options->getData()->hue;

    // change name if changed
    if (strcmp(selchnl->name,selector->getData()->name)) {
      if (selchnl->name) delete selchnl->name;
      selchnl->name= new char[strlen(selector->getData()->name)+1];
      strcpy(selchnl->name,selector->getData()->name);
      emit newChannelListSIGNAL(&channel_head);
    }

    emit channelChangedSIGNAL(selchnl);
  } else {
    // add new channel
    // FIX ME
    /*
    channel *n= new channel(&channel_head,0,"empty",DCOL,DBRI,DHUE,DCONTRAST,DNORM,DIN,
			    CFV_ACTUAL,0,selchnl->number+1);
			    */
    //debug("new number %d, %d",selchnl->number,selchnl->count());
    channel *n= new channel(&channel_head,
			    selector->getData()->frequency,
			    selector->getData()->name,
			    options->getData()->color,
			    options->getData()->brightness,
			    options->getData()->hue,
			    options->getData()->contrast,
			    selector->getData()->norm,
			    selector->getData()->input);
    selchnl=n;

    //debug("new number %d",n->number);

    selchnl->startwiththis=selector->getData()->startwiththis;
    if (selchnl->startwiththis) {
      channel *nn;
      nn=&channel_head;
      while (nn->next) {
	nn=nn->next;
	if (nn != selchnl) {
	  nn->startwiththis=0;
	}
      }
    }

    emit newChannelListSIGNAL(&channel_head);
    emit channelChangedSIGNAL(selchnl);
  }
}

// =============================================================================== 

void winTVMainWidget::applyMtabSLOT()
{
  if (rc->sound_on) {
    mix->set_vol(rc->volume_left,rc->volume_right);
  } else {
    mix->set_vol(0,0);
  }
}

// =============================================================================== 

void winTVMainWidget::saveChannelsSLOT()
{
  saveChannels((const char*) rc->channelListName);
}

// =============================================================================== 

void winTVMainWidget::saveChannelsAsSLOT()
{
  QString fileName = 
    QFileDialog::getSaveFileName(QDir::homeDirPath()+QString("/")+QString(RCDIR),"*.ch",this,filename);
  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);
    saveChannels(filename); 
  }
}

// =============================================================================== 

void winTVMainWidget::loadChannelsSLOT()
{
  QString fileName = 
    QFileDialog::getOpenFileName(QDir::homeDirPath()+QString("/")+QString(RCDIR),"*.ch",this);
  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);
    loadChannels(filename); 
  }
}

// =============================================================================== 

void winTVMainWidget::importXAWTVChannelsSLOT()
{
  char tag[32], val[100];
  char line[255], dummy[255];
  country cntry;

  QString fileName = QFileDialog::getOpenFileName(QDir::homeDirPath (),"*xawtv",this);

  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);

    QFile fin(filename);

    if (!fin.open(IO_ReadOnly)) {
      warning(klocale->translate("main: Can't load channel-filename %s!"),filename);
      if (channel_head.next==NULL) {
	selchnl= new channel(&channel_head,rc->freq,"<no channel>",rc->col,rc->bri,rc->hue,rc->contr,rc->norm,rc->in);
      }
      //emit newChannelListSIGNAL(&channel_head);
      //if (selchnl != NULL) emit channelChangedSIGNAL(selchnl);
      return;
    }

    QTextStream t( &fin ); 

    channel *n;
    // remove existing channels 
    if (channel_head.next != NULL) {
      n=&channel_head;
      // deleting the head deletes all channels
      n->~channel();
      channel_head.next=NULL;
      
#ifdef mainDEBUG
      debug("main: removing channel list");
#endif
    }

    int thenorm=DNORM;
    int theinput=DIN;

    strcpy(line,(const char *)t.readLine());
    while(line[0] != '[' ) {
//debug("xawtv: line: <%s>",line);
      if ((strlen(line)==0) || line[0] == '#' || line[0] == '%') {
	strcpy(line,(const char *)t.readLine());
	continue;
      }
      if (2 != sscanf(line," %31[^= ] = %99[^\n]",tag,val)) {
	warning("main: xawtv: parse error %s:%s in %s\n",tag,val,filename);
      } 

//debug("xawtv: scan: %s:%s",tag,val);

      if ( 0 == strcmp(tag,"freqtab")) {
        cntry.change(val);
//debug("xawtv: country %s %s",tag,val);
      } else if ( 0 == strcmp(tag,"source") ) {
	theinput= atoi(val);
      } else if (0 == strcmp(tag,"norm")) {
	thenorm= atoi(val);
      } else if (0 == strcmp(tag,"fullscreen")) {
	warning("main: scan xawtv: tag <%s> ignored!",tag);
      } else if (0 == strcmp(tag,"mixer")) {
	warning("main: scan xawtv: tag <%s> ignored!",tag);
      }
      strcpy(line,(const char *)t.readLine());
    }

//debug("xawtv: channel scan");
    selchnl=NULL;
    
    while(!t.eof()) {
      n= new channel(&channel_head,0,"<no name>",DCOL,DBRI,DHUE,DCONTRAST,thenorm,theinput);
      
      if (n->name) delete(n->name);
      n->name=new char[strlen(line)+1];
      strncpy(n->name,&line[1],strlen(line)-1);
      n->name[strlen(line)-2]=0;
//debug("xawtv: channel name: %s, %s",n->name,line);

      strcpy(line,(const char *)t.readLine());
      while (!t.eof() && (line[0] != '[')) {
	if ((strlen(line)==0) || line[0] == '#' || line[0] == '%') {
	  strcpy(line,(const char *)t.readLine());
	  continue;
	}
	if (2 != sscanf(line," %31[^= ] = %99[^\n]",tag,val)) {
	  warning("main: xawtv: parse error <%s>:<%s> in %s\n",tag,val,filename);
	}
//debug("xawtv: scan: %s:%s",tag,val);
	if (0 == strcmp(tag,"key")) { 
	  warning("main: scan xawtv: tag <%s> ignored!",tag);
	} else if (0 == strcmp(tag,"capture")) {
	  warning("main: scan xawtv: tag <%s> ignored!",tag);
	} else if (0 == strcmp(tag,"source")) {
	  n->input = 0;
	} else if (0 == strcmp(tag,"norm")) {
	  n->norm = 0;
	} else if (0 == strcmp(tag,"channel")) {
	  n->frequency = cntry.cftofreq(val,0);
//debug("xawtv: frequency <%s>, %d",val,n->frequency);
	} else if (0 == strcmp(tag,"fine")) {
	  warning("main: scan xawtv: tag <%s> ignored!",tag);
	} else if (0 == strcmp(tag,"color"))
	  n->color = atoi(val);
	else if (0 == strcmp(tag,"bright"))
	  n->brightness = atoi(val);
	else if (0 == strcmp(tag,"hue"))
	  n->hue = atoi(val);
	else if (0 == strcmp(tag,"contrast"))
	  n->contrast = atoi(val);
	else if (0 == strcmp(tag,"freqtab"))
	  warning("main: scan xawtv: tag <%s> ignored!",tag);
	strcpy(line,(const char *)t.readLine());
      }

#ifdef mainDEBUG
      debug("main: channel: %s, %d, %d",n->name, n->frequency, n->frequency*100/16);
#endif
    
    }

    selchnl=n;
    emit newChannelListSIGNAL(&channel_head);
    emit channelChangedSIGNAL(selchnl);

  }
}

// =============================================================================== 

void winTVMainWidget::importXTVSCREENChannelsSLOT()
{
  QString fileName = QFileDialog::getOpenFileName(QDir::homeDirPath (),"*.*",this,".xtvrc");
  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);
    channel_head.import(true);
    loadChannels(filename); 
    channel_head.import(false);
  }
}


// =============================================================================== 

void winTVMainWidget::IRCoptionsSLOT()
{
#ifdef HAVE_IRC
  naSLOT();
#else 
  naSLOT();
#endif
}

// =============================================================================== 

void winTVMainWidget::vtxSLOT()
{
  QMessageBox::information( this, 
			    (const char*)(QString(klocale->translate("VTX for kwintv "))+QString(VERSION)),
                            klocale->translate("Not aviable!"));
}

// =============================================================================== 

void winTVMainWidget::naSLOT()
{
  QMessageBox::information( this, 
			    (const char*)(QString("kwintv ")+QString(VERSION)),
                            klocale->translate("This function is not aviable in this Version!"));

}

// =============================================================================== 

void winTVMainWidget::saveRCSLOT()
{
  if (rc->save()) 
    QMessageBox::warning( this, 
			  (const char*)(QString("kwintv ")+QString(VERSION)),
			  klocale->translate("kwintv was not able to save rc data!"));
}

// =============================================================================== 

void winTVMainWidget::toggleMenuBarSLOT()
{
  /*
  if (menubar->isVisible()) menubar->hide();
  else menubar->show();
  updateRects();
  */
}

// =============================================================================== 

void winTVMainWidget::toggleToolBarSLOT()
{
  enableToolBar();
}

// =============================================================================== 

void winTVMainWidget::propertiesSLOT()
{
  int i;

  struct video_capability* vcap=winTVScreen->getCapCap();
  struct video_channel* vchan=winTVScreen->getCapChannel();
  struct video_audio* vaudio=winTVScreen->getCapAudio(); 
  struct video_tuner* vtuner=winTVScreen->getCapTuner();

  QString propstr(2000);
  QString dummy;

#ifdef mainDEBUG  
  debug("%s",vcap->name);
#endif
  propstr+=dummy.sprintf("Informations about your frame grabber card\n\n");
  propstr+=dummy.sprintf("Name: %s\n\nCapabilities:\n\t",vcap->name);
  for (i = 0; device_cap[i] != NULL; i++)
    if (vcap->type & (1 << i))
      propstr+=dummy.sprintf("%s  ",device_cap[i]);

  // channels 
  propstr+=dummy.sprintf("\n\nNumber of Channels: %d\n",vcap->channels);
  for (i = 0; i < vcap->channels; i++) {
    propstr+=dummy.sprintf("\t %s: %d %s %s%s%s\n",
					 vchan[i].name,
					 vchan[i].tuners,
					 (vchan[i].flags & VIDEO_VC_TUNER)   ? "tuner "  : "tuner ",
					 (vchan[i].flags & VIDEO_VC_AUDIO)   ? "audio "  : "",
					 (vchan[i].type & VIDEO_TYPE_TV)     ? "tv "     : "",
					 (vchan[i].type & VIDEO_TYPE_CAMERA) ? "camera " : "");
  }

  // tuner 
  propstr+=dummy.sprintf("\nType of Tuner: %s \n\t Range: %lu - %lu MHz\n\tSupported norms: ",
	vtuner->name,vtuner->rangelow,vtuner->rangehigh);
  for (i = 0; norms[i].str != NULL; i++) 
    if (vtuner->flags & (1<<i)) {
      propstr+=dummy.sprintf("%s %s  ",norms[i].str,(vtuner->mode == i)?"(aktive)":"");
    }

  // audios
  propstr+=dummy.sprintf("\n\nNumber of Audios: %d",vcap->audios);
  for (i = 0; i < vcap->audios; i++) {
    propstr+=dummy.sprintf("\n\t %d (%s): \n",i,vaudio[i].name);
    if (vaudio[i].flags & VIDEO_AUDIO_MUTABLE)
      propstr+=dummy.sprintf("\t\t mutable, muted = %s   ",(vaudio[i].flags&VIDEO_AUDIO_MUTE) ? "yes":"no");
    if (vaudio[i].flags & VIDEO_AUDIO_VOLUME)
      propstr+=dummy.sprintf("volume = %d  ",vaudio[i].volume);
    else 
      propstr+=dummy.sprintf("no volume   ");
    if (vaudio[i].flags & VIDEO_AUDIO_BASS)
      propstr+=dummy.sprintf("bass = %d   ",vaudio[i].bass);
    if (vaudio[i].flags & VIDEO_AUDIO_TREBLE)
      propstr+=dummy.sprintf("treble = %d   ",vaudio[i].treble);
    propstr+=dummy.sprintf("active mode is %s",audiomode[vaudio[i].mode].str);
  }

  QMessageBox::information( this, 
			    (const char*)(QString(klocale->translate("Properties of kwintv "))+QString(VERSION)),
                            (const char*)propstr);

}

// =============================================================================== 

void winTVMainWidget::splitSlidersSLOT()
{
#ifdef mainDEBUG
  debug("main: splitSlidersSLOT %d",rc->sliders_att);
#endif
  emit toggleSplitSlidersSIGNAL();
  smenu->setItemChecked(smid[6], rc->sliders_att); 
}

// =============================================================================== 

void winTVMainWidget::soundSLOT()
{
  rc->sound_on=!rc->sound_on;

#ifdef mainDEBUG
  debug("main: soundSLOT: %s",rc->sound_on?"on":"off");
#endif
  
  if (rc->sound_on) {
    mix->set_vol(rc->volume_left,rc->volume_right);
  } else {
    mix->set_vol(0,0);
  }
  emit audioMuteToggledSIGNAL();
  smenu->setItemChecked(smid[5], !rc->sound_on ); 
}

// =============================================================================== 

void winTVMainWidget::volumeSLOT(int vl,int vr)
{
  if ((rc->volume_left != vl) || (rc->volume_right != vr)) {
#ifdef mainDEBUG
    debug("main: volumeSLOT %d, %d",vl,vr);
#endif
    rc->volume_left=vl;
    rc->volume_right=vr;
    
    if (rc->sound_on) {
      mix->set_vol(rc->volume_left,rc->volume_right);
    }
    
    emit volumeChangedSIGNAL(rc->volume_left,rc->volume_right);
  }
}

// =============================================================================== 

void winTVMainWidget::volumeUpSLOT()
{
  int vl=rc->volume_left+MIXER_STEP;
  int vr=rc->volume_right+MIXER_STEP;
  if (vl > MIXER_MAX_VALUE) vl=MIXER_MAX_VALUE;
  if (vr > MIXER_MAX_VALUE) vr=MIXER_MAX_VALUE;

  volumeSLOT(vl,vr);
}

// =============================================================================== 

void winTVMainWidget::volumeDownSLOT()
{
  int vl=rc->volume_left-MIXER_STEP;
  int vr=rc->volume_right-MIXER_STEP;
  if (vl < 0) vl=0;
  if (vr < 0) vr=0;

  volumeSLOT(vl,vr);
}

// =============================================================================== 

void winTVMainWidget::channelUpSLOT()
{
  if (selchnl->prev!=&channel_head) {
    selchnl=selchnl->prev;
  } else {
    while (selchnl->next) { selchnl=selchnl->next; }
  }
#ifdef mainDEBUG
  debug("main: channel up: %s",selchnl->name);
#endif
  emit channelChangedSIGNAL(selchnl);
}

// =============================================================================== 

void winTVMainWidget::channelDownSLOT()
{
  if (selchnl->next) {
    selchnl=selchnl->next;
  } else {
    selchnl=channel_head.next;
  }
#ifdef mainDEBUG
  debug("main: channel down: %s",selchnl->name);
#endif
  emit channelChangedSIGNAL(selchnl);
}

// =============================================================================== 

void winTVMainWidget::loadChannels(const char *FILE)
{
  channel *n,*h;
  int count;
  bool add;

  ifstream fin(FILE);

  if (!fin) {
    warning(klocale->translate("main: Can't load channel-file %s!"),FILE);
    if (channel_head.next==NULL) {
      selchnl=new channel(&channel_head,rc->freq,"<no channel>",rc->col,rc->bri,rc->hue,rc->contr,rc->norm,rc->in);
    }
    emit newChannelListSIGNAL(&channel_head);
    if (selchnl != NULL) emit channelChangedSIGNAL(selchnl);
    return;
  }

  // remove existing channels 
  if (channel_head.next != NULL) {
    count=channel_head.count();
    n=&channel_head;
    // deleting the head deletes all channels
    n->~channel();
    channel_head.next=NULL;

#ifdef mainDEBUG
    debug("main: removing channel list");
#endif
  }

  selchnl=NULL;

#ifdef mainDEBUG
    debug("main: loading channels from %s",FILE);
#endif

  while(fin.ignore(1000, '*')!=NULL){
    n= new channel(&channel_head,0,"<no name>",DCOL,DBRI,DHUE,DCONTRAST,DNORM,DIN);

    fin >> *n;
    h=&channel_head;
    add=TRUE;
    while(h->next){
      h=h->next;
      // do we already have that frequency ?
      if(!strcmp(h->name,n->name) && n!=h){ 
        n->~channel();
        add=FALSE;      
        warning(klocale->translate("main: Can't load existing channel: %s"),h->name);
      }
    }
    if(add){
      if (n->startwiththis) {
	selchnl=n;
      }
    }
#ifdef mainDEBUG
      debug("main: channel: %s, %d, %d",n->name, n->frequency, n->frequency*100/16);
#endif
  }
  
  if (selchnl==NULL) {
    selchnl=channel_head.next;
  }

  emit newChannelListSIGNAL(&channel_head);
  if (selchnl != NULL) emit channelChangedSIGNAL(selchnl);

#ifdef mainDEBUG
      debug("main: channel count: %d",channel_head.count());
#endif
}

// =============================================================================== 

void winTVMainWidget::saveChannels(const char *FILE)
{
  channel *h;
  
  ofstream fout(FILE);
  if (!fout) {
    warning(klocale->translate("main: Can't save channel-file %s!"),FILE);
    return;
  }
  
  fout << "#" << endl;
  fout << "# kwintv Channel File" << endl;
  fout << "#" << endl;

  h=&channel_head;
  
  while(h->next){ 
    h=h->next;
    fout << *h << endl;
  }

  chown(FILE,getuid(),getgid());

#ifdef mainDEBUG
  debug("main: channel-file saved!");
#endif
}


// =================================================================================================

void winTVMainWidget::showChannelInfoSLOT(channel *ch)
{
  if ( !isVisible() || (channelLCDpos==NONE) ) return;

  //switch_capture(0);
  channelLCD->show();

  int Xoffset,Yoffset;

  switch (FRAME_LEFT) {
  case FRAME_NONE:
    Xoffset=0;
    Yoffset=0;
    break;
  case FRAME_RIGHT:
  case FRAME_LEFT:
    Xoffset=toolbar->width();
    Yoffset=0;
    break;
  case FRAME_DOWN:
  case FRAME_UP:
    Xoffset=0;
    Yoffset=toolbar->height();
    break;
  default:
    warning(klocale->translate("main: position not supported!"));
  }

  switch (channelLCDpos) {
  case UPPER_LEFT:
    channelLCD->setGeometry(geometry().x()-20,geometry().y()+20,
			    width()/8,height()/8);
    break;
  case UPPER_RIGHT:
    channelLCD->setGeometry(geometry().x()+width()-(width()/8)-20,geometry().y()+20,
			    width()/8,height()/8);
    break;
  case LOWER_LEFT:
    channelLCD->setGeometry(geometry().x()-20,geometry().y()+height()-(height()/8)-20,
			    width()/8,height()/8);
    break;
  case LOWER_RIGHT:
    channelLCD->setGeometry(geometry().x()+width()-(width()/8)-20,
			    geometry().y()+height()-(height()/8)-20,
			    width()/8,height()/8);
    break;
  }

  channelLCD->display(ch->number);
  channelLCD->raise();
  
  if (infoTimer->isActive()) infoTimer->stop();
  infoTimer->start(LCD_DELAY,TRUE);
  
}

//==============================================================================

void winTVMainWidget::unShowChannelInfoSLOT()
{
  if (channelLCD->isVisible()) {
    channelLCD->hide();
  }
}

// =================================================================================================

void winTVMainWidget::resizeEvent( QResizeEvent * theEvent)
{
  if (channelLCD->isVisible()) channelLCD->hide();

  if (rc->autoar) adjustAspectRatioSLOT();
  else fixed=FALSE;

  if (isVisible()) KApplication::getKApplication()->sendEvent(winTVScreen,theEvent);

  updateRects();

#ifdef mainDEBUG
  debug("main: resize");
#endif
}

// =================================================================================================

void winTVMainWidget::moveEvent( QMoveEvent * theEvent)
{
  if (channelLCD->isVisible()) channelLCD->hide();

  if (isVisible()) KApplication::getKApplication()->sendEvent(winTVScreen,theEvent);
  
#ifdef mainDEBUG
  debug("main: move");
#endif
}

// =================================================================================================
/*
void winTVMainWidget::updateRects( )
{
  //KApplication::getKApplication()->sendEvent(winTVScreen,theEvent);
  
#ifdef mainDEBUG
  debug(klocale->translate("main: update"));
#endif
}
*/

// =================================================================================================

void winTVMainWidget::emitSetupSignals()
{
  emit newChannelListSIGNAL(&channel_head);
  emit volumeChangedSIGNAL(rc->volume_left,rc->volume_right);
  if (selchnl != NULL) emit channelChangedSIGNAL(selchnl);
}

// =================================================================================================
// =================================================================================================

#include "wintvmain.moc"






  
