/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// Hey, better do not change the order of the includes !!!
// REMEMBER: Qt-stuff BEFORE X11-stuff !!!

#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include <strstream.h>
#include <unistd.h>
#include <linux/string.h> // for memcpy
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>

#include "wintvscreen.h"

#include <qframe.h> 
#include <qcolor.h>
#include <qpixmap.h>

// video clip stuff
#include "avivideoclip.h"
#include "rawvideoclip.h"
#include "ppmvideoclip.h"

// includes needed for X stuff 
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/IntrinsicP.h>
#include <X11/ObjectP.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>

#ifdef HAVE_LIBXXF86DGA
#include <X11/extensions/xf86dga.h>
#endif

#include "qt_visual.h"

#ifdef X11DEBUG
#include "xeventtype.h"
#endif

#define HEX(N) hex << setw(N) << setfill(int('0')) 
#define DEC(N) dec << setw(N) << setfill(int('0')) 

int one=1;
int zero=0;

//==============================================================================
//==============================================================================
// einige X errors muessen (sollten) ignoriert werden,
// z.b. BadWindow und so.
// Qt wuerde sonst mit fatal(...) aussteigen...
//
XErrorHandler xerror(Display *dpy, XErrorEvent *event)
{
  switch (event->error_code){
  case BadWindow:
#ifdef tvDEBUG
    warning("screen: X error BadWindow ingnored");
#endif
    return 0 ;
    break;
      
  case BadMatch:
#ifdef tvDEBUG
    warning("screen: X error BadMatch ingnored");
#endif
    return 0 ;
    break;

  case BadColor:
#ifdef tvDEBUG
    warning("screen: X error BadColor ingnored");
#endif
    return 0 ;
    break;

  default:
#ifdef tvDEBUG
    warning("screen: unknown X error %d ingnored",event->error_code);
#endif
    return 0;
  }
}

//============================= winTVScreenWidget =================================================
//============================= winTVScreenWidget =================================================

// local variables to avoid inclusion of X11 header stuff in wintvscreen.h
XColor xcolors[256];
Colormap xcolormap;       // colormap, needed when having only 256 colors -> pseudocolor

winTVScreenWidget::winTVScreenWidget( QWidget *parent, 
				      const char *name,
				      int _capture, 
				      int _automouseoff,
				      char *_bttvdev, 
				      int _bpp,int _dual,
				      int _freq, 
				      int _input, 
				      char *_visualname 
				      )
  : QWidget( parent, name ), 
    FREQ(_freq), 
    INPUT(_input), 
    CAP_ON(_capture), 
    autoMouseOFF(_automouseoff),
    devname(_bttvdev),
    visname(_visualname),
    bpp(_bpp),
    dual(_dual)
{ 
#ifdef tvDEBUG
  debug("screen: begin");
#endif

    // do some widget stuff
    //setCaption("xWinTV - TV Screen");
    //    setBackgroundcolor
    setMinimumSize( 10, 10 );
    setMaximumSize(32000,32000);
    //resize(win_default_width(),win_default_height());

    QColor bkgc(0,0,0);
    setBackgroundColor(bkgc); 
    
    /*
    // set background pixmap
    pm->load((const char*)(QString(ICONPATH)+QString("tv2_large.xpm")));
    setBackgroundPixmap(*pm);
    */

    /* obsolete or FIX ME
    // enable mouse tracking
    if (!hasMouseTracking()) setMouseTracking(TRUE);
    // timer for automatic deactivation of mouse pointer
    mtimer = new QTimer(this);
    connect(mtimer,SIGNAL(timeout()),this,SLOT(changeCursorStateSLOT()));
    */

    // timer for autodetection of audio mode
    audioDetectTimer = new QTimer(this);
    connect(audioDetectTimer,SIGNAL(timeout()),this,SLOT(autoDetectAudioModeSLOT()));

#ifndef NOCARD
    // init everything (bttv)
#ifdef tvDEBUG
    debug("screen: initialize");
#endif 

    // set up device
    if ( -1 == (fbttv=open(devname,O_RDWR)) ) {
      fatal("screen: Can not open the bttv-device <%s>",devname);
    }

    // use my XErrorHandler 
    XSetErrorHandler((XErrorHandler) xerror);

    initializeTV();

    // final init of bttv
    // unmute, audio mode = auto
    /*
    vaudio[0].flags &= ~VIDEO_AUDIO_MUTE;
    if (-1 == ioctl(fbttv,VIDIOCSAUDIO,&vaudio[0]))
      fatal("ioctl VIDIOCSAUDIO");
      */

    /* set win */
    if(ioctl(fbttv, VIDIOCSWIN, &vwin))
      warning("ioctl VIDIOCSWIN");
    /* Set the input */
    if(ioctl(fbttv, VIDIOCSCHAN, &INPUT))
      fatal("screen: set input");
    /* Set the frequency */
    if(ioctl(fbttv, VIDIOCSFREQ, &FREQ))
      fatal("screen: set frequency");

    if (CAP_ON){
      Change=TRUE;
      newTVGeometry();
      captureState=1;
      // enabel/disable capture
      if(ioctl(fbttv, VIDIOCCAPTURE, &one))
	fatal("screen: capture on");
    } else {
      Change=FALSE;
      if(ioctl(fbttv, VIDIOCCAPTURE, &zero))
	fatal("screen: capture off");
    }
     
#endif // NOCARD 

    // install filter for special X events
    installEventFilter( this );

    // we are running
    is_aktive=1;

#ifdef tvDEBUG
  debug("screen: end");
#endif
}

//==============================================================================

winTVScreenWidget::~winTVScreenWidget()
{
  if (is_aktive) destroyTV();
  //debug("blubb");
}

//==============================================================================
// besondere X11 events, die nicht von Qt direkt abgefangen werden.

bool winTVScreenWidget::x11Event( XEvent *event )
{
  static bool UNMAPPED=FALSE;
  XWindowAttributes wts1,wts2;
  Window checkw1,checkw2,wwin;
  Display *displ;
  int x,x2,y,y2;
  uint ww,wh;

#ifdef X11DEBUG
  debug("screen: own x11Event %s",events[event->type]);
#endif

  if ((!isVisible())||(wwin=(Window) winId()) == None) return(FALSE);

  displ=qt_xdisplay();
  Fresh=TRUE;
  switch_capture(0);

  checkw1=event->xmap.window;
  checkw2=event->xany.window;

#ifdef X11DEBUG
  // debug("x11Event: id %d",checkw2);
#endif

  if (CAP_ON)
    { 
      if ( wwin==checkw1 || wwin==checkw2 ) Fresh=FALSE;
      else {
	if (checkw1){ 
	  XGetWindowAttributes(displ, checkw1, &wts1);

	  wh=vwin.height; ww=vwin.width;
	  
	  if (!(wts1.map_state & isVisible()))
	    Fresh=FALSE;
	  
	  x=wts1.x-vwin.x; y=wts1.y-vwin.y;
	  x2=x+wts1.width+2*wts1.border_width-1;
	  y2=y+wts1.height+2*wts1.border_width-1;
	  if ((x2 < 0) || (x >= (int)ww) ||
	      (y2 < 0) || (y >= (int)wh))
	    Fresh=FALSE;
	}
	if (checkw2) {
	  XGetWindowAttributes(displ, checkw2, &wts2);

	  wh=vwin.height; ww=vwin.width;
	  
	  if (!(wts2.map_state & isVisible()))
	    Fresh=FALSE;
	  
	  x=wts2.x-vwin.x; y=wts2.y-vwin.y;
	  x2=x+wts2.width+2*wts2.border_width-1;
	  y2=y+wts2.height+2*wts2.border_width-1;
	  if ((x2 < 0) || (x >= (int)ww) ||
	      (y2 < 0) || (y >= (int)wh))
	    Fresh=FALSE;
	}
      }
    }
 
#ifdef X11DEBUG
  //debug("screen: x11Event: Fresh==%d",Fresh);
#endif

  switch (event->type) {
  case VisibilityNotify:
    switch (event->xvisibility.state) {

    case VisibilityUnobscured:
#ifdef X11DEBUG
    debug("screen: VisibilityNotify:VisibilityUnobscured");
#endif
      if (CAP_ON || wasCaptureOn)
	{
	  if (wasCaptureOn) CAP_ON=TRUE;
	  wasCaptureOn=FALSE;
	  if (checkw2 != DefaultRootWindow(displ)) {
	    Change=TRUE; 
	    getTVClip();
	  }
	}
      break;

    case VisibilityFullyObscured:
#ifdef X11DEBUG
      debug("screen: VisibilityNotify:FullyObscured");
#endif
      if (checkw2 != DefaultRootWindow(displ)) {
	wasCaptureOn=CAP_ON;
	capture(0);
      }
      //maybe:
      //capture(0);
      break;

    case VisibilityPartiallyObscured:
#ifdef X11DEBUG
    debug("screen: VisibilityNotify:VisibilityPartiallyObscured");
#endif
      if (CAP_ON || wasCaptureOn)
	{
	  if (wasCaptureOn) CAP_ON=TRUE;
	  wasCaptureOn=FALSE;
	  if (checkw2 != DefaultRootWindow(displ)) {
	    Change=TRUE;
	    getTVClip();
	  }
	}
      break;
    }
    break;
    
    //  case 10:
  case MapNotify:
#ifdef X11DEBUG
    debug("screen: MapNotify");
#endif
    if ((DefaultRootWindow(displ) != checkw2) && UNMAPPED){
      UNMAPPED=FALSE;
      CAP_ON=TRUE;
      switch_capture(1);
      if ( Fresh ){
	refreshTV();
      }
    }
    if ((CAP_ON) ){
      if ( Fresh ){
	refreshTV();
      }
    }
    break;


  case ConfigureNotify:
#ifdef X11DEBUG
    debug("screen: ConfigureNotify");
#endif
    //switch_capture(0);
    getTVClip();
    break;

  case Expose:
#ifdef X11DEBUG
    debug("screen: Expose");
#endif
    //switch_capture(0);
    //newTVGeometry();
    break;


  case DestroyNotify:
#ifdef X11DEBUG
      debug("screen: DestroyNotify");
#endif

    if (wwin  == checkw2){
      switch_capture(0);
    }
    break;

    //  case 9:
  case UnmapNotify:
#ifdef X11DEBUG
    debug("screen: UnmapNotifyNotify");
#endif
    if (DefaultRootWindow(displ) != checkw2){
      switch_capture(0);
      if (CAP_ON) UNMAPPED=TRUE;
      if (CAP_ON) {
	if ( Fresh ){
	  refreshTV();
	}
      }
      CAP_ON=FALSE;
    } else
    if (wwin  != event->xmap.window)
      if(CAP_ON) {
	getTVClip(); 
      }
    break;

  case EnterNotify:
    if (CAP_ON && CMSET){ 
      //XInstallColormap(displ,xcolormap);
#ifdef tvDEBUG
      debug("screen: EnterNotify (set colormap)");
#endif
      }
      break;
      
  case LeaveNotify:
    // Looks better this way 
    if (CAP_ON && CMSET){ 
      //XUninstallColormap(displ,xcolormap);
#ifdef tvDEBUG
    debug("screen: LeaveNotify (unset colormap)");
#endif
    }
    break;

#ifdef X11DEBUG
  default:
    debug("x11Event: no event found: %d",event->type);
#endif
  }

  if (CAP_ON) {
   switch_capture(1); 
  }


  return (FALSE);
}

//==============================================================================
// X11 events, die vom root window (i. e. desktop) erzeugt werden, z.B.
// MapNotify etc.

bool winTVScreenWidget::eventFilter( QObject *, QEvent *e )
{
  static bool UNMAPPED=FALSE;
  //XWindowAttributes wts1,wts2;
  Window wwin;
  Display *displ;

  if ( e->type() == XRWEvent ) {

    if ((!isVisible())||(wwin=(Window) winId()) == None) return(FALSE);

    X11RWEvent *re = (X11RWEvent*)e;

#ifdef X11DEBUG
    debug("screen: xrwevent %s",events[re->XEventTyp()]);
#endif

    displ=qt_xdisplay();
    Fresh=TRUE;
    switch_capture(0);

#ifdef X11DEBUG
    //debug("screen: eventFilter: Fresh=%d",Fresh);
#endif

    Fresh=FALSE;
    switch (re->XEventTyp()) {
    case MapNotify:
      if (UNMAPPED){
	UNMAPPED=FALSE;
	if (CAP_ON) {
	  if (Fresh) 
	    refreshTV();
	}
      }
      break;
      
    case ConfigureNotify:
      if (CAP_ON) {
	getTVClip();
      }
      break;
      
    case DestroyNotify:
      if (CAP_ON) {
	//getTVClip();
      }
      break;
      
    case UnmapNotify:
      if (CAP_ON) UNMAPPED=TRUE;
      if (CAP_ON) {
	getTVClip();
	if ( Fresh ){
	  refreshTV();
	}
      }
      //  ??? CAP_ON=FALSE;
      break;
      
    } // switch
    
    switch_capture(1);

    return TRUE;                        // eat event
  }

  return FALSE;                         // standard event processing
}

//==============================================================================

void winTVScreenWidget::moveEvent ( QMoveEvent * theEvent )
{
  Fresh=TRUE;
  Change=FALSE;
  //Change=TRUE;
  Move=TRUE;
#ifndef NOCARD
  switch_capture(0);
  newTVGeometry();
  switch_capture(1);
#endif
#ifdef tvDEBUG
  debug("screen: move: %d, %d",theEvent->pos().x(),theEvent->pos().y());
#endif
}

//==============================================================================

void winTVScreenWidget::resizeEvent( QResizeEvent * theEvent )
{
  Fresh=TRUE;
  Change=FALSE;
  //Change=TRUE;
  Move=FALSE;
#ifndef NOCARD
  switch_capture(0);
  newTVGeometry();
  switch_capture(1);
#endif 
#ifdef tvDEBUG
  debug("screen: resize: %d, %d",theEvent->size().width(),theEvent->size().height());
#endif
}

//==============================================================================
/*
void winTVScreenWidget::paintEvent( QPaintEvent * )
{
  Fresh=TRUE;
  //Change=FALSE;
  Change=TRUE;
  Move=FALSE;
#ifndef NOCARD
  switch_capture(0);
  getTVClip();
  switch_capture(1);
#ifdef tvDEBUG
  ioctl(fbttv, BTTV_GETWTW, &(wtw));
  debug("paint x:%d y:%d xd:%d yd:%d ",wtw.x,wtw.y,wtw.width,wtw.height);
#endif
#endif
}
*/
//==============================================================================

void winTVScreenWidget::keyPressEvent ( QKeyEvent *keyE )
{
  emit keyPressedSIGNAL(keyE->key());
  keyE->ignore();
}

//==============================================================================
/*
void winTVScreenWidget::mouseMoveEvent(QMouseEvent *me)
{
#ifdef tvDEBUG
    debug("screen: mouseMoveEvent");
#endif

  if (cursorState) {
    if (mtimer->isActive()) mtimer->stop();
    mtimer->start(MOUSE_DELAY,TRUE);
#ifdef tvDEBUG
    debug("screen: timer started");
#endif
  } else {
    changeCursorStateSLOT();
  }
		     
}
*/
// =============================================================================== 

void winTVScreenWidget::mouseReleaseEvent ( QMouseEvent * me)
{
#ifdef mainDEBUG
  debug("screen: mouseReleaseEvent");
#endif
  if ((me->button() == RightButton) && (tvPopup != NULL)) {
    tvPopup->popup(mapToGlobal(me->pos()));
  }
}


//==============================================================================

void winTVScreenWidget::switch_capture(int on)
{
  if (isVisible()) {
    if (on && CAP_ON) {
      if(ioctl(fbttv, VIDIOCCAPTURE, &one))
	fatal("screen: sw capture on");
    }
    else {
      if(ioctl(fbttv, VIDIOCCAPTURE, &zero))
	fatal("screen: sw capture off");
    }
  }
}

//==============================================================================

void winTVScreenWidget::capture(int on)
{
#ifdef tvDEBUG
  debug("screen: capture=%s",on?"on":"off");
#endif

  if (isVisible()) {
    if (on) {
      Fresh=TRUE;
      //Change=FALSE;
      Change=TRUE;
      Move=TRUE;
      CAP_ON=TRUE; 
      newTVGeometry();

      if(ioctl(fbttv, VIDIOCCAPTURE, &one))
	fatal("screen: capture on");
    }
    else {
      if(ioctl(fbttv, VIDIOCCAPTURE, &zero))
	fatal("screen: capture off");
      CAP_ON=FALSE;
      repaint();
    }
  }
}
//==============================================================================

//==============================================================================

int winTVScreenWidget::initializeTV()
{
  Window rootw;
  //int col;
  // uint rwidth;  // why unit ??
  int rwidth;
  int visual_class;
  int major, minor, bank, ram;
  Display *disp;
  ushort i,r,rr;
  Colormap cmap;
  Visual *myvisual;
  //  XSetWindowAttributes swinattr;

#ifndef NOCARD

  // set up v4l
  if(ioctl(fbttv,VIDIOCGCAP,&vcap))
    fatal("screen: capabilitys");
#ifdef tvDEBUG
  debug("\t name: %s",vcap.name);
  for (i = 0; device_cap[i] != NULL; i++)
    if (vcap.type & (1 << i))
      debug("\t %s",device_cap[i]);
#endif  
  
  /* channels */
#ifdef tvDEBUG
  debug("screen: channels: %d",vcap.channels);
#endif  
  vchan = (struct video_channel*)malloc(sizeof(struct video_channel)*vcap.channels);
  memset(vchan,0,sizeof(struct video_channel)*vcap.channels);
  for (i = 0; i < vcap.channels; i++) {
    vchan[i].channel = i;
    if (ioctl(fbttv,VIDIOCGCHAN,&vchan[i]))
      fatal("screen: get input channels");
#ifdef tvDEBUG
    debug("\t %s: %d %s%s %s%s",
	  vchan[i].name,
	  vchan[i].tuners,
	  (vchan[i].flags & VIDEO_VC_TUNER)   ? "tuner "  : "",
	  (vchan[i].flags & VIDEO_VC_AUDIO)   ? "audio "  : "",
	  (vchan[i].type & VIDEO_TYPE_TV)     ? "tv "     : "",
	  (vchan[i].type & VIDEO_TYPE_CAMERA) ? "camera " : "");
#endif
  }
  
  /* audios */
#ifdef tvDEBUG
  debug("screen: audios: %d",vcap.audios);
#endif

  vaudio = (struct video_audio*)malloc(sizeof(struct video_audio)*vcap.audios);
  memset(vaudio,0,sizeof(struct video_audio)*vcap.audios);
  for (i = 0; i < vcap.audios; i++) {
    vaudio[i].audio = i;
    if (-1 == ioctl(fbttv,VIDIOCGAUDIO,&vaudio[i]))
      fatal("ioctl VIDIOCGCAUDIO");
#ifdef tvDEBUG
    debug("\t %d (%s): ",i,vaudio[i].name);
    if (vaudio[i].flags & VIDEO_AUDIO_MUTABLE)
      debug("\t mutable, muted=%s ",(vaudio[i].flags&VIDEO_AUDIO_MUTE) ? "yes":"no");
    if (vaudio[i].flags & VIDEO_AUDIO_VOLUME)
      debug("\t volume=%d ",vaudio[i].volume);
    if (vaudio[i].flags & VIDEO_AUDIO_BASS)
      debug("\t bass=%d ",vaudio[i].bass);
    if (vaudio[i].flags & VIDEO_AUDIO_TREBLE)
      debug("\t treble=%d ",vaudio[i].treble);
    debug("\t mode=%d",vaudio[i].mode);
    if (vaudio[i].flags & VIDEO_AUDIO_VOLUME) debug("\t can volume, %d",vaudio[i].volume);
#endif
  }
  audioVolumeCan=(vaudio[0].flags & VIDEO_AUDIO_VOLUME);
  audioMode=vaudio[0].mode;
  //audioVolume=vaudio[0].volume;
  audioVolume=127;
  
  if(ioctl(fbttv, VIDIOCGTUNER, &(vtuner)) == -1)
    //fatal("screen: get tuner");
    warning("screen: get tuner");
#ifdef tvDEBUG
  debug("screen: tuner   : %s %lu-%lu",
	vtuner.name,vtuner.rangelow,vtuner.rangehigh);
  for (i = 0; norms[i].str != NULL; i++) 
    if (vtuner.flags & (1<<i)) {
      debug("\t %s %s",norms[i].str,(vtuner.mode == i)?"*":"");
    }
#endif
  if(ioctl(fbttv, VIDIOCGWIN, &(vwin)) == -1)
    fatal("screen: get window");
  if(ioctl(fbttv, VIDIOCGFBUF, &(vbuf)) == -1)
    fatal("screen: get frame buffer");

  // this should be done by bttv itself
  vwin.width=DEFAULT_WIDTH;
  vwin.height=DEFAULT_HEIGHT;
  vwin.x=0;
  vwin.y=0;
  vwin.clipcount=0;
  rwidth=vbuf.width;

  disp=qt_xdisplay();

  // do some strange things with visuals and dga
  int flags;
  void *base = NULL;

#ifdef HAVE_LIBXXF86DGA
  if (XF86DGAQueryVersion(disp, &major, &minor)) { 
    XF86DGAQueryDirectVideo(disp,XDefaultScreen(disp),&flags);
    if (flags & XF86DGADirectPresent) {
      XF86DGAGetVideoLL(disp, DefaultScreen(disp),(int *)&base,
			&rwidth, &bank, &ram);
      if (!base) fatal("screen: can not allocate video buffer base adress: 0x%x",base);
    }
  }
#endif /* HAVE_LIBXXF86DGA */

  // FIX ME visuals
  // get the visual and check for doublebuffering
  if (!dual) {
    dual=find_visual(disp,visname,&myvisual,&visual_class);
  }
  // FIX MEX
  if (dual) vbuf.depth=32;

#ifdef HAVE_LIBXXF86DGA
  if ( vbuf.base != base ) {
    warning("v4l and dga disagree about the framebuffer address: %d, need %d.",vbuf.base,base);
    vbuf.base=(void *)base;
    if( -1 == ioctl(fbttv, VIDIOCSFBUF, &vbuf)) 
      fatal("must be root to set frame buffer");
  }
  if ( bpp ) {
    vbuf.depth= bpp;
    if( -1 == ioctl(fbttv, VIDIOCSFBUF, &vbuf)) 
      fatal("must be root to set colour depth");
  } else 
  if ( vbuf.depth != QColor::numBitPlanes() ) {
    warning("v4l and dga disagree about the colour depth: %d bpp, need %d bpp.",vbuf.depth,QColor::numBitPlanes());
    vbuf.depth=QColor::numBitPlanes();
    if( -1 == ioctl(fbttv, VIDIOCSFBUF, &vbuf)) 
      fatal("must be root to set colour depth");
  }
  // FIX ME Test auf richtige screen size
  // do we need that??
  if ( vbuf.bytesperline != rwidth*vbuf.depth/8 ) {
    warning("v4l and dga disagree about the number of bytes per line: %d, need %d.",vbuf.bytesperline,rwidth*vbuf.depth/8);
    vbuf.bytesperline=rwidth*vbuf.depth/8;
    if( -1 == ioctl(fbttv, VIDIOCSFBUF, &vbuf)) 
      fatal("must be root to set number of bytes per line");
  }
#endif /* HAVE_LIBXXF86DGA */

  numbitplanes=QColor::numBitPlanes();

  // Alloc memory for snapshot 
  if (0 == strncmp(vcap.name,"BT848",5)) {
    msize = 2*BTTV_MAX_FBUF; //768*576*4; /* largest possible picture*/
    TargetMem=(char *)mmap(0,msize,PROT_READ|PROT_WRITE,MAP_SHARED,fbttv,0);
    if ((char*)-1 == TargetMem)
      warning("screen: unable to allocate memory for snap shots and video clips");
  } else  TargetMem = (char*)-1;

#ifdef tvDEBUG
  debug("screen: visual %s, dualmode %s",visname,dual?"ON":"OFF");
  debug("screen: bpp %d, bpl %d, width %d, height %d",vbuf.depth,vbuf.bytesperline,vwin.width,vwin.height);
  debug("screen: video buffer base adress: 0x%x",vbuf.base);
  debug("screen: memory for snap shots: 0x%08x", (int) TargetMem);
#endif

#endif /* NOCARD */

  CMSET=FALSE;
  // we must use PseudoColor, i.e. we have only 256 colors
  if (visual_class == PseudoColor) {
#ifdef tvDEBUG
    debug("screen: using color class PseudoColor");
#endif
    CMSET=TRUE;
    for (i=0;i<256;i++) {
      xcolors[i].pixel=i;
      xcolors[i].flags = DoRed | DoGreen | DoBlue;
    }
    xcolormap=DefaultColormap(disp,DefaultScreen(disp));
    XQueryColors(disp, xcolormap, xcolors, 256);
    xcolormap=XCreateColormap(disp, DefaultRootWindow(disp),myvisual, AllocAll);
    for (i=0;i<225;i++){ // Set Colors 16 - 240 
      r=i%25;
      xcolors[i+16].green =(65535*(i/25)/8);  
      xcolors[i+16].red =(65535*(r/5)/4);  
      rr=r%5;
      xcolors[i+16].blue =(65535*rr/4);        
    }
    XStoreColors(disp,xcolormap,xcolors,256);
    XSetWindowColormap(disp,(Window)winId(),xcolormap);
#ifdef tvDEBUG
    debug("screen: set up colormap for 8 bpp");
#endif
  }
  /*
  int value;
  // we must use PseudoColor, i.e. we have only 256 colors
  if (visual_class == PseudoColor) {
    CMSET=TRUE;
    alloc_context = QColor::enterAllocContext();
#ifdef tvDEBUG
      debug("screen: ac: %d",alloc_context);
#endif
    for (i=0;i<225;i++){ // Set Colors 16 - 240 
      r=i%25;rr=r%5;
      qcolors[i+16]=QColor( (255*(r/5)/4), (255*(i/25)/8),(255*rr/4) ); 
      //  qcolors[i+16]=QColor(  (65535*(r/5)/4), (65535*(i/25)/8),(65535*rr/4) );    
      value=qcolors[i+16].pixel();
#ifdef tvDEBUG
      //debug("screen: rgb: %d %d %d",(65535*(r/5)/4), (65535*(i/25)/8),(65535*rr/4));
      //debug("screen: p: %d, rgb: %d %d %d",value,qcolors[i+16].red(),qcolors[i+16].green(),qcolors[i+16].blue());
#endif
    }

#ifdef tvDEBUG
    debug("screen: set up colormap for 8 bpp");
#endif
  }
  */

  // set up some X stuff

  rootw=DefaultRootWindow(disp);

#ifdef tvDEBUG
  debug("screen: rootwindow id: %d",rootw);
#endif

  // set XEvents we want to receive
  const uint stdWidgetEventMask =                 // X event mask
        KeyPressMask | KeyReleaseMask |
        ButtonPressMask | ButtonReleaseMask |
        KeymapStateMask |
        ButtonMotionMask |
        EnterWindowMask | LeaveWindowMask |
        FocusChangeMask |
        ExposureMask | VisibilityChangeMask |
        StructureNotifyMask | SubstructureNotifyMask;

  XSelectInput(disp,(Window) winId(), 
	       stdWidgetEventMask );


  XSelectInput(disp,rootw, 
	       ExposureMask | VisibilityChangeMask | 
	       StructureNotifyMask | SubstructureNotifyMask );


  // init some variables
 
  REFRESH=TRUE;
  Move=FALSE;
  Fresh=FALSE;
  Change=FALSE;
  DELAY=DEFAULT_DELAY;
  wasCaptureOn=FALSE;

  saverState=1;
  cursorState=1;
  muteState=0;
  captureState=0;
  showchannelState=1;

  timer_inactive=TRUE;

  shotnumber=0;
  video_aktive=0;
  video_capturing=0;

  setTVMaxSize();
}

//==============================================================================
/*
int winTVScreenWidget::getCap(int unit, int lun, int cap)
{
  switch (unit) {
    

  }
  
  return (0);
}
*/
//==============================================================================

void winTVScreenWidget::setTVMaxSize()
{
  uint max_h,max_w;
 switch (NORM) {
  case PAL:
  case SECAM:
    max_w=PAL_WIDTH;
    max_h=PAL_HEIGHT;
    break;

  case NTSC:
    max_w=NTSC_WIDTH;
    max_h=NTSC_HEIGHT;
    break;
    
  default:
    max_w=DEFAULT_WIDTH;
    max_h=DEFAULT_HEIGHT;
    break;
  }

  max_width=max_w;
  max_height=max_h;

#ifdef tvDEBUG
  debug ("screen: screensize max_x: %d, max_y: %d",max_width,max_height);
#endif
}

//==============================================================================

void winTVScreenWidget::destroyTV()
{
  int error;

  if (is_aktive) {
    capture(0);
    if (!saverState) toggleScreenSaverSLOT();
    
    is_aktive=0;
    
    if (CMSET) {
      XFreeColormap(qt_xdisplay(),xcolormap);
      //QColor::leaveAllocContext (); 
      //QColor::destroyAllocContext ( alloc_context );
    }
    
#ifndef NOCARD
    /*
    if (-1 == ioctl(fbttv,VIDIOCSYNC,0))
      perror("ioctl VIDIOCSYNC");
      */

#ifdef tvDEBUG
    debug("screen: freeing mem");
#endif
    if ((char*)-1 != TargetMem)
      munmap(TargetMem,BTTV_MAX_FBUF*2); 
#endif

#ifdef tvDEBUG
    debug("screen: closing device %s",devname);
#endif
    // close device
    error=close(devname);
    /*
      if (error != 0) {
      warning("error %d closing device %s",error,devname);
      error=close(devname);
    }
    */

#ifdef tvDEBUG
    debug("screen: winTVScreenWidget destroyed");
#endif

  }

}

//==============================================================================

void winTVScreenWidget::newTVGeometry(int x, int y, int w, int h)
{
#ifndef NOCARD
  int xp,yp,the_width,the_height;
  int dx,dy;
  bool CHECK;
  int oldx,oldy,oldw,oldh;

#ifdef tvDEBUG
  debug("screen: newTVGeometry");
#endif

  if (!isVisible()) return;

  oldx=vwin.x;
  oldy=vwin.y;
  oldw=vwin.width;
  oldh=vwin.height;
  CHECK=Change;
  vwin.flags=0;

  // get new geometry values, need the global position 
  QPoint p(0,0);
  p=mapToGlobal(p);
  xp=p.x();
  yp=p.y();
  the_height=geometry().height();
  the_width=geometry().width();
 
#ifdef tvDEBUG
  debug("screen: newTVGeometry: the_height %d, the_width %d",the_height,the_width);
#endif

  // check if size changed
  if(oldh!=the_height) CHECK=TRUE;
  if(oldw!=the_width) CHECK=TRUE;

  dx=0;dy=0;

  // set a new width
  if (the_width > max_width) {
    dx=(the_width - max_width)/2;
#ifdef tvDEBUG
  debug("screen: newTVGeometry: dx %d",dx);
#endif
    if (oldw != max_width) {
      CHECK=TRUE;
      vwin.width=max_width & ~3;
    }
  } else {
    vwin.width=the_width & ~3;
  }

  // set a new height
  if (the_height > max_height) {
    dy=(the_height - max_height)/2;
#ifdef tvDEBUG
  debug("screen: newTVGeometry: dy %d",dy);
#endif
    if (oldh != max_height) {
      CHECK=TRUE;
      vwin.height=max_height & ~3;
    }
  } else {
    vwin.height=the_height & ~3;
  }

  // check if position changed
  if (oldx!=xp+dx) {
    Move=TRUE;
    CHECK=TRUE;
  }
  if (oldy!=yp+dy) {
    Move=TRUE;
    CHECK=TRUE;
  }

  // FIX ME
  // why do we do that ???
  if ((vbuf.depth==24)&&((xp+dx)&3)) 
    {
      xp=xp-((xp+dx) &3);
    }
  else if ((vbuf.depth==16)&&((xp+dx)&1))
    {
      xp=xp-((xp+dx) &1);
    } 
  else if ((vbuf.depth==15)&&((xp+dx)&1))
    {
      xp=xp-((xp+dx) &1);
    }

  if ((yp+dy)&1) 
    {
      yp=yp-((yp+dy) &1);
    }

  if (CHECK) {
    vwin.x=xp+dx;
    vwin.y=yp+dy;
    if (CAP_ON) {
#ifdef tvDEBUG
      debug("screen: newTVGeometry x %d, y %d , w %d, h %d, dx %d, dy %d",vwin.x,vwin.y,vwin.width,vwin.height,dx,dy);
#endif
      if( -1 == ioctl(fbttv, VIDIOCSWIN, &vwin))
	fatal("screen: set clipped window");
    }
  }  

  if (CAP_ON && CHECK) getTVClip(); 

  if(!Change && CAP_ON && Fresh) {
    refreshTV();
    Fresh=FALSE;
  }
  
  Change=FALSE;
#endif
}

//==============================================================================

void winTVScreenWidget::autoDetectAudioModeSLOT()
{
  if (-1 == ioctl(fbttv,VIDIOCGAUDIO,&vaudio[0]))
    fatal("ioctl VIDIOCGAUDIO");

  emit autoDetectAudioModeSIGNAL(vaudio[0].mode | AUDIOMODE_AUTODETECT);

#ifdef tvDEBUG
  //debug("screen: autoDetectAudioModeSLOT mode: %s",audiomode[vaudio[0].mode].str);
#endif
}

//==============================================================================

void winTVScreenWidget::refreshTVActionSLOT()
{
#ifndef NOCARD
  int scr,depth;
  XSetWindowAttributes xswa;
  unsigned long mask;
  Window rwin,rootW;
  Display *disp;
  

#ifdef tvDEBUG
    debug("screen: refreshTVAction");
#endif
  if (REFRESH){
    disp=qt_xdisplay();
    scr=DefaultScreen(disp);
    rootW= RootWindow(disp, scr);
    
    depth=numbitplanes;
    
    xswa.override_redirect = TRUE;
    xswa.backing_store = NotUseful;
    xswa.save_under = FALSE;
    xswa.background_pixel=BlackPixel(disp,scr);
    xswa.backing_store=NotUseful;
    mask = (CWBackPixel | CWSaveUnder | CWBackingStore|
	    CWOverrideRedirect );
    if(Move){
      rwin = XCreateWindow(disp,rootW,0, 0, 
			   vbuf.width,vbuf.height, 0,
			   CopyFromParent, InputOutput, CopyFromParent,
			   mask, &xswa);
      Move=FALSE;
    } else {
      rwin = XCreateWindow(disp,rootW,
			   vwin.x-80, vwin.y-80, 
			   vwin.width+80,vwin.height+80,
			   0,CopyFromParent, InputOutput, CopyFromParent,
			   mask, &xswa);
    }
    //    lastrwin=rwin;
    XMapWindow(disp, rwin);
    XUnmapWindow(disp, rwin);
    XDestroyWindow(disp, rwin);
  }
#endif
  timer_inactive=TRUE;
}

//==============================================================================


void  winTVScreenWidget::refreshTV()
{
#ifdef tvDEBUG
  debug("screen: refreshTV");
#endif
  if (timer_inactive) {
    QTimer::singleShot(DELAY,this,SLOT(refreshTVActionSLOT()));
    timer_inactive=FALSE;
    }
}

//==============================================================================

void winTVScreenWidget::addTVClip(struct video_clip *cr,int *ncr,uint x,uint y,uint x2,uint y2)
{
#ifndef NOCARD
  cr[*ncr].x=x;
  cr[*ncr].y=y;
  cr[*ncr].width=x2-x;
  cr[*ncr].height=y2-y;
  *ncr=*ncr+1;
#endif

#ifdef X11DEBUG
  debug("screen: addTVClip: added clip %d, %d, %d, %d",x,y,x2,y2);
#endif
} 

//==============================================================================

void winTVScreenWidget::makeTVClipTab(int ncr)
{
#ifndef NOCARD
  vwin.clips=cliprecs;
  vwin.clipcount=ncr;
  
  if( -1 == ioctl(fbttv, VIDIOCSWIN, &vwin) )
    warning("screen: VIDIOCSWIN");
#endif
}

//==============================================================================

#ifndef NOCARD
int winTVScreenWidget::getTVClip(void)
{
  int ncr=0;
  int x,y,x2,y2,wx,wy;
  uint ww,wh;
  Display *disp;
  XWindowAttributes wts;
  Window parent,win,parent2, root, rroot, *children, swin;
  uint nchildren, i;
  struct video_clip *cr; 

  if (!isVisible()) return(0);

  disp=qt_xdisplay();

  /*
  widpar=(Widget) w;
  while(XtParent(widpar)) widpar=XtParent(widpar);
  if((win=XtWindow(widpar))==None) return;
  */

  win=(Window)winId();
  XQueryTree(disp,win, &rroot, &parent, &children, &nchildren);
  if(nchildren) XFree((char *)children);
 
  cr=cliprecs;

  wx=vwin.x; wy=vwin.y;
  wh=vwin.height; ww=vwin.width;

  //#if 0
  if (wx<0)
    addTVClip(cr, &ncr, 0, 0, (uint)(-wx), wh-1);
  if (wy<0)
    addTVClip(cr, &ncr, 0, 0, ww-1, (uint)(-wy));
  if ((wx+ww) > vbuf.width)
    addTVClip(cr, &ncr, vbuf.width-wx, 0, ww-1, wh-1);
  if ((wy+wh) > vbuf.height)
    addTVClip(cr, &ncr, 0, vbuf.height-wy, ww-1, wh-1);
  //#endif
    
  root=DefaultRootWindow(disp);
  swin=win;
  while (parent!=root) {
    swin=parent;
    XQueryTree(disp, swin, &rroot, &parent, &children, &nchildren);
    if (nchildren)
      XFree((char *) children);
  }
  XQueryTree(disp, root, &rroot, &parent2, &children, &nchildren);
  for (i=0; i<nchildren;i++) 
    if (children[i]==swin) break;
  i++;

#ifdef X11DEBUG
  debug("screen: getTVClip: nchildren %d, %d",nchildren,i);
#endif

  for (; i<nchildren; i++) {
    
    XGetWindowAttributes(disp, children[i], &wts);
    if (!(wts.map_state /*& isVisible()*/))  continue;
    
    x=wts.x-vwin.x; 
    y=wts.y-vwin.y;
    //x2=x+wts.width+2*wts.border_width-1;
    //y2=y+wts.height+2*wts.border_width-1;
    x2=x+wts.width+2*wts.border_width;
    y2=y+wts.height+2*wts.border_width;
    if ((x2 < 0) || (x >= (int)ww) ||
	(y2 < 0) || (y >= (int)wh))
      continue;

    if (x<0)
      x=0;
    if (y<0)
      y=0;
    if (x2>=(int)ww)
      x2=ww-1;
    if (y2>=(int)wh)
      y2=wh-1;
    addTVClip(cr, &ncr, x, y, x2, y2);
  }
  XFree((char *) children);

  makeTVClipTab(ncr);
  return ncr;
}
#endif

//==============================================================================

void flip(char *mem, int len)
{
  int i;
  char tmp;

  for (i=0;i<len;i++) {
    tmp=mem[2];
    mem[2]=mem[0];
    mem[0]=tmp;
    mem+=3;
  }
}

//==============================================================================
//---------------------------------- SLOTS -------------------------------------
//==============================================================================

void winTVScreenWidget::snapShotSLOT(int FULL, int count)
{
#ifndef NOCARD
  
  char name[80];
  char *tm=TargetMem;

  if ((tm != (char *)-1) && CAP_ON && !video_aktive) {
    if (FULL){
      vgrab_even.width=max_width;
      vgrab_even.height=max_height;
    } else {
      vgrab_even.width=vwin.width;
      vgrab_even.height=vwin.height;
    }

    vgrab_even.format=BT848_COLOR_FMT_RGB24;
    vgrab_even.frame=0;

    ioctl(fbttv, VIDIOCMCAPTURE, &vgrab_even);
    ioctl(fbttv, VIDIOCSYNC, 0);

    // dont overwrite existing files...
    bool file_exists=TRUE;
    while (file_exists) {
      name[0]=0;
      ostrstream oname(name,80);
      oname << PICNAME << HEX(3) << shotnumber << ".pnm" << char(0);
      ifstream os_t(name);
      if (! os_t) {
	file_exists=FALSE;
	os_t.close();
      } else {
	shotnumber++;
	os_t.close();
      }
    }

    ostrstream oname(name,80);
    oname << PICNAME << HEX(3) << shotnumber << ".pnm" << char(0);
    ofstream os(name);
    
    os << "P6\n" << dec << vgrab_even.width << "\n"
      << dec << vgrab_even.height << "\n255\n";

    // FIX ME do we need this??
    //flip(tm, vgrab_even.width*vgrab_even.height);
    os.write(tm,vgrab_even.width*vgrab_even.height*3);
    os.close();
    chown(name,getuid(),getgid());

    //shotnumber++;

  } // CAP_ON
#endif // nocard
}

//==============================================================================

void winTVScreenWidget::startVideoClipSLOT(struct MOVIE_PARAMS *movieparams)
{
#ifndef NOCARD
  if ((TargetMem != (char *)-1) && !video_aktive) {
    video_aktive=1;

    switch (movieparams->video_clip) {
    case VIDEOCLIP_AVI: // avi
      video= new aviVideoClip();
      break;
    case VIDEOCLIP_PPM: // ppm
      video= new ppmVideoClip();
      break;
    case VIDEOCLIP_RAW: // raw
      video= new rawVideoClip();
      break;
    default:
      fatal("screen: unsupported video format %d",movieparams->video_clip);
    }

    // open video-file
    video->open(movieparams->filename,movieparams);

#ifdef tvDEBUG
    debug("screen: opened video-file %s",movieparams->filename);
#endif

    // map video_format to corresponding BT848 format
    switch (video->parameter()->video_format) {
    case VIDEO_RGB08:
	vgrab_even.format = BT848_COLOR_FMT_RGB8;
	break;
    case VIDEO_RGB15:
	vgrab_even.format = BT848_COLOR_FMT_RGB15;
	break;
    case VIDEO_RGB16:
	vgrab_even.format = BT848_COLOR_FMT_RGB16;
	break;
    case VIDEO_RGB24:
	vgrab_even.format = BT848_COLOR_FMT_RGB24;
	break;
    case VIDEO_RGB32:
	vgrab_even.format = BT848_COLOR_FMT_RGB32;
	break;
    case VIDEO_GRAY8:
	vgrab_even.format = BT848_COLOR_FMT_Y8;
	break;
    case VIDEO_YCrCb_422:
	vgrab_even.format = BT848_COLOR_FMT_YUY2;
	break;
    case VIDEO_YCrCb_411:
	vgrab_even.format = BT848_COLOR_FMT_BtYUV;
	break;
    case VIDEO_YCrCb_422p:
	vgrab_even.format = BT848_COLOR_FMT_YCrCb422;
	break;
    case VIDEO_YCrCb_411p:
	vgrab_even.format = BT848_COLOR_FMT_YCrCb411;
	break;
    default:
	fatal("screen: unsupported video format");
    }

    // set up vgrab
    vgrab_even.frame=0;
    vgrab_even.width=movieparams->width;
    vgrab_even.height=movieparams->height;

    memcpy(&vgrab_odd,&vgrab_even,sizeof(struct video_mmap));

    vgrab_odd.frame=1;

    video_frame_even=1;

#ifdef tvDEBUG
    debug("screen: starting video timer %dms, %d fps",1000 / video->parameter()->fps,video->parameter()->fps);
    debug("screen: size: %dx%d",vgrab_even.width,vgrab_even.height);
#endif

    // timer starten
    video_capturing=0;
    
    videoTime.start();
    // FIX ME
    // wie genau ist der timer??????????????????????????????
    writeVideoClipFrameSLOT();
    //videoTimer->start(1000 / video->parameter()->fps);
  }
#endif
}

//==============================================================================
// looped version
void winTVScreenWidget::writeVideoClipFrameSLOT()
{
#ifndef NOCARD
  long frames=1;
  long lostframes=0;
  long fps=video->parameter()->fps;
  bool lost=FALSE;

  emit lostVideoClipFrameSIGNAL(lostframes);

  videoTime.restart();

  while (video_aktive) {
    if ( (((double)(frames * 1000) / videoTime.elapsed()) > fps) ) {
      qApp->processEvents( 20 ); 
    } else {
      ioctl(fbttv, VIDIOCMCAPTURE,video_frame_even ? &vgrab_even : &vgrab_odd);
      ioctl(fbttv, VIDIOCSYNC, 0);
      
      video->writeframe(video_frame_even ? (void*)(TargetMem) : (void*)(TargetMem + BTTV_MAX_FBUF));
      //video.writesound(...);
      frames++;
      emit wroteVideoClipFrameSIGNAL(video->nframes());
      video_frame_even=!video_frame_even;
      lost=FALSE;
      elapsed=videoTime.elapsed();
      while ( ((double)(frames * 1000) / elapsed) < fps ) {
	// lost a frame!
	lostframes++;
	frames++;
	lost=TRUE;
      }
      if (lost) emit lostVideoClipFrameSIGNAL(lostframes);
    }
  }
  elapsed=videoTime.elapsed();
#endif // nocard
}
//processEvents ( int maxtime ) 
//==============================================================================

void winTVScreenWidget::endVideoClipSLOT()
{
#ifndef NOCARD
  long frames;

  if (video_aktive) {
    video_aktive=0;
    frames=video->nframes();
    // close and delete videowriter
    video->close();
    //delete video;
    emit wroteVideoClipFrameSIGNAL(video->nframes());
    emit statVideoClipFrameSIGNAL((double)frames/elapsed*1000);
#ifdef tvDEBUG
    debug("screen: closed video-file");
    debug("screen: wrote %d frames, %d msec. , %3.1f fps",frames,elapsed,(double)frames/elapsed*1000);
#endif
  }
#endif // nocard
}

//==============================================================================

void winTVScreenWidget::changeFrequencySLOT(int value)
{
#ifndef NOCARD

  if ( FREQ != value ) {
    FREQ=value;
    if( -1 == ioctl(fbttv, VIDIOCSFREQ, &FREQ) )
      warning("screen: change frequency");
  }
  
#ifdef tvDEBUG
  debug("screen: changeFrequencySLOT %d",value);
#endif

#endif
}

//==============================================================================

void winTVScreenWidget::changeCaptureSLOT(int value)
{
  captureState=!captureState;
  /*  if (value) {
    CAP_ON=TRUE;
    newTVGeometry();
  }*/
  capture(value);
}

//==============================================================================

void winTVScreenWidget::changeBrightnessSLOT(int bri)
{
#ifdef tvDEBUG
  debug("screen: changeBrightnessSLOT %d",bri);
#endif

#ifndef NOCARD
  struct video_picture vp;

  if (Bright != bri)
    {
      if(ioctl(fbttv, VIDIOCGPICT, &vp)==-1)
	warning("screen: get picture params");
      Bright=bri;
      vp.brightness=(Bright+128)<<8;
      if(ioctl(fbttv, VIDIOCSPICT, &vp))
	fatal("screen: set picture params");
    }
#endif // NOCARD
}

//==============================================================================

void winTVScreenWidget::changeContrastSLOT(int contr)
{
#ifdef tvDEBUG
  debug("screen: changeContrastSLOT %d",contr);
#endif

#ifndef NOCARD
  struct video_picture vp;

  if (Contrast != contr)
    {
      if(ioctl(fbttv, VIDIOCGPICT, &vp)==-1)
	warning("screen: get picture params");
      Contrast=contr;
      vp.contrast=Contrast<<7;
      if(ioctl(fbttv, VIDIOCSPICT, &vp))
	fatal("screen: set picture params");
    }
#endif
}

//==============================================================================

void winTVScreenWidget::changeColorSLOT(int color)
{
#ifdef tvDEBUG
  debug("screen: changeColorSLOT %d",color);
#endif

#ifndef NOCARD
  struct video_picture vp;

  if (Color != color)
    {
      if(ioctl(fbttv, VIDIOCGPICT, &vp)==-1)
	warning("screen: get picture params");
      Color=color;
      vp.colour=Color<<7;
      if(ioctl(fbttv, VIDIOCSPICT, &vp))
	fatal("screen: set picture params");
    }
#endif
}

//==============================================================================

void winTVScreenWidget::changeHueSLOT(int hue)
{
#ifdef tvDEBUG
  debug("screen: changeHueSLOT %d",hue);
#endif

#ifndef NOCARD
  struct video_picture vp;

  if (Hue != hue)
    {
      if(ioctl(fbttv, VIDIOCGPICT, &vp)==-1)
	warning("screen: get picture params");
      Hue=hue;
      vp.hue=(Hue+128)<<8;
      if(ioctl(fbttv, VIDIOCSPICT, &vp))
	fatal("screen: set picture params");
    }
#endif
}

//==============================================================================

void winTVScreenWidget::changeSourceSLOT(int s)
{
#ifdef tvDEBUG
  debug("screen: changeSourceSLOT %d",s);
#endif

#ifndef NOCARD
  if (INPUT != s){
    INPUT=s;
    if(ioctl(fbttv, VIDIOCSCHAN, &(INPUT)))
      fatal("screen: change tuner");
  }
#endif
}

//==============================================================================

void winTVScreenWidget::changeNormSLOT(int n)
{
#ifdef tvDEBUG
  debug("screen: changeNormSLOT %d",n);
#endif

#ifndef NOCARD
  if (n != NORM)
    {
     vtuner.mode=n;
     NORM=n;
     Change=TRUE;
     setTVMaxSize();
     if (NORM==NTSC) Fresh=TRUE;
     newTVGeometry();
     if (ioctl(fbttv, VIDIOCSTUNER, &vtuner))
       fatal("screen: change norm");
    }

#endif
}

//==============================================================================

void winTVScreenWidget::changeDelaySLOT(int d)
{
#ifndef NOCARD

#ifdef tvDEBUG
  debug("screen: changeDelaySLOT %d",d);
#endif

  if (DELAY != d) DELAY=d;

#endif
}

//==============================================================================

void winTVScreenWidget::changeChannelValuesSLOT(channel *c)
{
  int pictmod=0;
  struct video_picture vp;

  if (!c) return;

#ifdef tvDEBUG
  debug("screen: change channel values: %s, %d",c->name,c->frequency);
#endif

#ifndef NOCARD
  if (CAP_ON) {
     switch_capture(0);
  }

  if (FREQ != c->frequency)
    {
      FREQ=c->frequency;
      if(ioctl(fbttv, VIDIOCSFREQ, &FREQ))
        warning("screen: change frequency");
    }
  
  /*
   *	Need to create a set format ioctl ??
   */
  
  if (NORM != c->norm)
    {
      NORM=c->norm;
      vtuner.mode=c->norm;
      Change=TRUE;
      setTVMaxSize();
      newTVGeometry();
      if (NORM==NTSC)
	Fresh=TRUE;
      newTVGeometry();
      if(ioctl(fbttv, VIDIOCSTUNER, &vtuner))
	warning("screen: change format");
    }
  
  if (INPUT != c->input)
    {
      INPUT=c->input;
      int v=INPUT;
      if(ioctl(fbttv, VIDIOCSCHAN, &v))
	warning("screen: change input");
    }
  
  if( -1 == ioctl(fbttv, VIDIOCGPICT, &vp) )
    warning("screen: get picture params");
  
  if (Bright != c->brightness)
    {
      Bright=c->brightness;
      vp.brightness=(Bright+128)<<8;
      pictmod=1;
    }
  
  if (Color !=c->color)
    {
      Color=c->color;
      vp.colour=c->color<<7;
      pictmod=1;
    }
  
  if (Hue !=c->hue)
    {
      Hue=c->hue;
      vp.hue=(c->hue+128)<<8;
      pictmod=1;
    }
  
  
  if (Contrast !=c->contrast)
    {
      Contrast=c->contrast;
      vp.contrast=c->contrast<<7;
      pictmod=1;
    }
  
  if(pictmod)
    {
      if(ioctl(fbttv, VIDIOCSPICT, &vp))
	warning("screen: set picture params");
    }  

  //  setCaption((const char*)(QString("xWinTV - ")+QString(c->name)));
  switch_capture(1);

#ifdef tvDEBUG
  debug("screen: changed values");
#endif

#endif 
}

//==============================================================================

void winTVScreenWidget::toggleAudioMuteSLOT()
{
  muteState=!muteState;

#ifdef tvDEBUG
  debug("screen: changeMuteSLOT, muted=%s",muteState?"yes":"no");
#endif
  
#ifndef NOCARD
  if (muteState)
    vaudio[0].flags |= VIDEO_AUDIO_MUTE;
  else
    vaudio[0].flags &= ~VIDEO_AUDIO_MUTE;

  vaudio[0].volume = audioVolume;

  if (-1 == ioctl(fbttv,VIDIOCSAUDIO,&vaudio[0]))
    fatal("ioctl VIDIOCSAUDIO");

#endif
}

//==============================================================================

void winTVScreenWidget::changeAudioVolumeSLOT(int vl, int vr)
{
#ifndef NOCARD

#ifdef tvDEBUG
    debug("screen: changeAudioVolumeSLOT set audio volume to %d, %d",vl,vr);
#endif
  if (!audioVolumeCan) return;

  if (muteState) vaudio[0].flags |= VIDEO_AUDIO_MUTE;
  else vaudio[0].flags &= ~VIDEO_AUDIO_MUTE;

  vaudio[0].volume = vl;
  audioVolume=vl;

  //vaudio[0].mode = audioMode ? audioMode : 0;

    if (-1 == ioctl(fbttv,VIDIOCSAUDIO,&vaudio[0]))
	fatal("ioctl VIDIOCSAUDIO");
#endif   
}

//==============================================================================

void winTVScreenWidget::changeAudioModeSLOT(int mode)
{
#ifndef NOCARD
  if ((mode > 0) && (mode < 5)) {
#ifdef tvDEBUG
    debug("screen: changeAudioModeSLOT set audio mode to %s",audiomode[mode].str);
#endif
    if (audioDetectTimer->isActive()) audioDetectTimer->stop();
    audioMode=audiomode[mode].nr;
    // try to set mode...
    vaudio[0].mode=audioMode;
    vaudio[0].volume = audioVolume;
    if (-1 == ioctl(fbttv,VIDIOCSAUDIO,&vaudio[0]))
      fatal("ioctl VIDIOCSAUDIO");
    if (-1 == ioctl(fbttv,VIDIOCGAUDIO,&vaudio[0]))
      fatal("ioctl VIDIOCGAUDIO");
    // set the real mode
    audioMode=vaudio[0].mode;
    // and inform others...
    emit autoDetectAudioModeSIGNAL(audioMode);
  } else {
    // autodetection, mode==0
    // set up timer
    audioMode=audiomode[mode].nr;
    if (audioDetectTimer->isActive()) audioDetectTimer->stop();
    audioDetectTimer->start(AUDIOMODE_DELAY);
  }
#endif 
}

//==============================================================================

void winTVScreenWidget::changeTVMultiDisplaySLOT(int x, int y, int w, int h)
{
#ifdef tvDEBUG
  debug("screen: changeTVMultiDisplaySLOT %d, %d, %d, %d",x,y,w,h);
#endif
}

//==============================================================================

void winTVScreenWidget::toggleScreenSaverSLOT()
{
  int t,i,p,a;
  Display *displ=qt_xdisplay();

  saverState=!saverState;
  if (saverState) {
#ifdef tvDEBUG
  debug("screen: enabled; saverState = %d",saverState);
#endif
    XGetScreenSaver(displ,
		    &t,&i,&p,&a);
    if (t == 0 && i == interval &&
	p == prefer_blanking && a == allow_exposures)
      XSetScreenSaver(displ,
		      timeout,interval,prefer_blanking,allow_exposures);
  } else {
#ifdef tvDEBUG
  debug("screen: disabled; saverState = %d",saverState);
#endif
    XGetScreenSaver(displ,
		    &timeout,&interval,&prefer_blanking,&allow_exposures);
    XSetScreenSaver(displ,
		    0,interval,prefer_blanking,allow_exposures);
  }

}

//==============================================================================

void winTVScreenWidget::toggleMouseCursorSLOT()
{
  cursorState=!cursorState;

#ifdef tvDEBUG
  debug("screen: cursorState = %d",cursorState);
#endif

  if (!cursorState) {
    orgCursor=cursor();
    setCursor(blankCursor);
  } else {
    setCursor(orgCursor);
  }
}


//==============================================================================
//==============================================================================
// never change the next line!
#include "wintvscreen.moc"

// end of file









