/* 
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <wintv.h>
#include <icondata.h>
#include <qfiledlg.h> 
#include <qmsgbox.h>

#include <kapp.h>
#include <klocale.h>

#include <wintvvclip.h>

#ifdef myDEBUG
#define vcDEBUG
#endif

#define SB_FRAMES 1
#define SB_FLOST  2
#define SB_FPS    3

/* ------------------------------------------------------------------------------*/
/* ------------------------------------------------------------------------------*/

winTVVClipWidget::winTVVClipWidget(QWidget* parent, const char* name, 
				   rcfile *_rc):  KTopLevelWidget( name ),rc(_rc)
{
#ifdef vcDEBUG
  debug("vc: begin of init");
#endif

  setCaption((const char*)(QString("kwintv - video clip")));
  // set icon
  QPixmap *pm=new QPixmap(xwintv_xpm);
  setIcon(*pm);
  setIconText("kwintv - video clip");

  /////////////////////////////////////////////////////////////////////////////
  statusbar= new KStatusBar(this,"statusbar");
  statusbar->insertItem("frames",SB_FRAMES);
  statusbar->insertItem("frames lost",SB_FLOST);
  statusbar->insertItem("frames per second",SB_FPS);

  setStatusBar(statusbar);

  /////////////////////////////////////////////////////////////////////////////
  QPopupMenu *filemenu=new QPopupMenu;
  filemenu->insertItem(klocale->translate("Save &As"),this,SLOT(saveAsSLOT()));
  filemenu->insertSeparator();
  filemenu->insertItem(klocale->translate("&Close"),this,SLOT(toggleVClipSLOT()));

  QPopupMenu *optionsmenu=new QPopupMenu;
  optionsmenu->insertItem(klocale->translate("Show/Hide Statusbar"),this,SLOT(toggleStatusBarSLOT()));
  optionsmenu->insertSeparator();
  optionsmenu->insertItem(klocale->translate("&Options"),this,SLOT(optionsSLOT()));

  // menu bar  
  KMenuBar *menubar = new KMenuBar( this, "kwintv -- MenuBar --" );
  menubar->insertItem(klocale->translate("&File"),filemenu);
  menubar->insertItem(klocale->translate("&Options"),optionsmenu);
  setMenu(menubar);

  /////////////////////////////////////////////////////////////////////////////
  vclip= new cvclip(this,"vclip",rc,&par);
  setView(vclip);

  /////////////////////////////////////////////////////////////////////////////
  par.video_format=rc->vc_format;
  par.width=rc->vc_width;
  par.height=rc->vc_height;
  par.fps=rc->vc_fps;
  if (rc->vc_audiocapture) par.channels=rc->vc_channels; 
  else par.channels=0;
  par.audio_format=rc->vc_audioformat;
  par.bits=rc->vc_bits;
  par.rate=rc->vc_rate;
  strcpy(par.filename,(const char *)rc->vc_tempName);
  par.video_clip=rc->vc_video_clip;

  /////////////////////////////////////////////////////////////////////////////
  // set up ctab dialogs
  tab= new vtab(NULL,"tab");
  vidaud= new vvidaud(tab,"vidaud");
  tab->addTab(vidaud,"Video/Audio");

  tab->setCancelButton();
  tab->setOkButton();
  tab->setDefaultButton();

  /////////////////////////////////////////////////////////////////////////////
  // connect everything
  connect(vclip,SIGNAL(closeSIGNAL()),this,SLOT(toggleVClipSLOT()));
  connect(vclip,SIGNAL(optionsSIGNAL()),this,SLOT(optionsSLOT()));
  connect(vclip,SIGNAL(runCaptureSIGNAL()),this,SLOT(vclipSLOT()));

  // connect ctab dialog apply, cancel, default
  connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyTabSLOT())); 
  connect(tab,SIGNAL(cancelButtonPressed()),this,SLOT(cancelTabSLOT())); 
  connect(tab,SIGNAL(defaultButtonPressed()),this,SLOT(defaultTabSLOT()));

  clip=TRUE;

  setMinimumSize( 200,100 );
  resize( 400,200 );
}

/* ------------------------------------------------------------------------------*/

winTVVClipWidget::~winTVVClipWidget()
{

}

/* ------------------------------------------------------------------------------*/

void winTVVClipWidget::toggleVClipSLOT()
{
  statusbar->changeItem("frames",SB_FRAMES);
  statusbar->changeItem("frames lost",SB_FLOST);
  statusbar->changeItem("frames per second",SB_FPS);
  if (!clip) {
    vclip->setTextSLOT("Start Capture");
    emit endVClipSIGNAL();
    clip=TRUE;
  }
  if (isVisible()) hide();
  else show();
}

/* ------------------------------------------------------------------------------*/

void winTVVClipWidget::optionsSLOT()
{
  par.channels=rc->vc_channels;
  vidaud->setData(&par);
  tab->show();
}

/* ------------------------------------------------------------------------------*/

void winTVVClipWidget::saveAsSLOT()
{
  QString fileName = 
    QFileDialog::getSaveFileName(".",QString("*")+QString(par.extension),this,NULL);
  if ( !fileName.isNull() ) {                 // got a file name
    // copy tmp avi file to file filename
    if (system((const char *)QString("mv ")+QString(par.filename)+QString(" ")+fileName))
      QMessageBox::warning( this, "kwintv",klocale->translate("Was unable to move file %s to file %s"),
			    par.filename,(const char *)fileName);  
  }
}

/* ------------------------------------------------------------------------------*/

void winTVVClipWidget::toggleStatusBarSLOT()
{
  enableStatusBar();
}

/* ------------------------------------------------------------------------------*/

void winTVVClipWidget::vclipSLOT()
{
#ifdef vcDEBUG
  debug("vc: video clip: %s",clip?"start":"stop");
#endif
  if (clip) {
    clip=FALSE;
    vclip->setTextSLOT("Stop Capture");
    emit startVClipSIGNAL(&par);
  } else {
    clip=TRUE;
    vclip->setTextSLOT("Start Capture");
    emit endVClipSIGNAL();
  }
  
}

// =============================================================================== 

void winTVVClipWidget::frameVClipSLOT(int c)
{
  statusbar->changeItem(dummy.setNum(c),SB_FRAMES);
}

// =============================================================================== 

void winTVVClipWidget::framelostVClipSLOT(int c)
{
  statusbar->changeItem(dummy.setNum(c),SB_FLOST);
}

void winTVVClipWidget::statVideoClipFrameSLOT(double d)
{
   statusbar->changeItem(dummy.setNum(d),SB_FPS);
}

void winTVVClipWidget::applyTabSLOT()
{
  if (rc->vc_audiocapture) par.channels=vidaud->getData()->channels;
  else par.channels=0;
  rc->vc_channels=vidaud->getData()->channels;
  par.fps=vidaud->getData()->fps;
  par.bits=vidaud->getData()->bits;
  par.rate=vidaud->getData()->rate;
  par.video_clip=vidaud->getData()->video_clip;
  strcpy(par.filename,vidaud->getData()->filename);
}
void winTVVClipWidget::defaultTabSLOT()
{
  vidaud->setData(&par);
}
void winTVVClipWidget::cancelTabSLOT()
{
  if (rc->vc_audiocapture) par.channels=rc->vc_channels; 
  else par.channels=0;
}
// =============================================================================== 
// =============================================================================== 
// =============================================================================== 
// never change the next line!
#include "wintvvclip.moc"

// end of file



